'''
@file: SeqAli.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2013

@ [oliva's lab](http://sbi.imim.es)

@class: SeqAli
@class: Rost
@class: SeqAliError
'''
import re
import decimal
import math

from SBI.sequence import Sequence
from SBI.beans    import IndexedNum


class SeqAli(object):
    '''
    Sequence alignment container.
    Capacity for multiple sequence alignment.
    Iterable. Iteration refers to the *alignment position*, which starts in *1*
    and counts also the gaps.
    A slice from a {SeqAli} also returns a {SeqAli} of the requested section.
    Capacity for *sequence positions* defined as {Integer} or {IndexedNum},
    which is useful for PDB sequence alignment.

    __DEVELOPER ADVISE:__ there is an intended difference between *alignment
    position* (that which refers to global alignment) and *sequence position*,
    which refers to the position of a given sequence according to where it
    begins. As a rule, everything works internally through *alignment position*.
    A request for a *sequence position* is internally transformed.

    '''
    _PATTERN_FORMATS = frozenset(['BLAST'])

    def __init__(self, sequences, sequence_inits,
                 identities = None, positives = None, gaps = None):
        '''
        @param:    sequences
        @pdef:     sequences to add to the alignment.
        @ptype:    {List} of {String} or {Sequence}

        @param:    sequence_inits
        @pdef:     initial number of each sequence of the alignment. (not all
                   sequences start alignment at 1)
        @ptype:    {List} of {Integer}

        @param:    identities
        @pdef:     number of identities in the alignment
        @pdefault: _None_
        @ptype:    {Integer}

        @param:    positives
        @pdef:     number of positives in the alignment
        @pdefault: _None_
        @ptype:    {Integer}

        @param:    gaps
        @pdef:     number of gaps in the alignment
        @pdefault: _None_
        @ptype:    {Integer}

        @raises: {AttributeError} if sequence or sequence_inits is not a {List}
                 or if they are not {List}s of the same length.
                 Also, if the sequence list contains something different than
                 {String} or {Sequence}
        @raises: {SeqAliError} if the fragmentation of the alignment encounters
                 some problem.
        '''
        if not isinstance(sequences, list):
            raise AttributeError('Sequences must be added in a list\n')
        if not isinstance(sequence_inits, list):
            raise AttributeError('Sequence inits must be added in a list\n')
        if len(sequences) != len(sequence_inits):
            raise AttributeError('One init is required for each sequence\n')

        self._error = SeqAliError()
        self._seq   = []

        for aliseq in sequences:
            if isinstance(aliseq, Sequence):
                self._seq.append(aliseq)
            elif isinstance(aliseq, basestring):
                self._seq.append(Sequence(sequence = aliseq))
            else:
                raise AttributeError('sequences must be specified as strings or Sequence objects.')

        self._num_seq = len(self._seq)
        self._segment = []

        self._idx = sequence_inits

        self._seq2ali  = self._built_seq2ali(sequence_inits)
        self._staticSA = self._seq2ali

        self._segmentation_ok = self._search_segments()

        if not self._segmentation_ok:
            raise self._error.wrong_segmentation()

        self._identities = identities
        self._positives  = positives
        self._gaps       = gaps
        self._alipatt    = None
        self._aliptmeth  = None

        self._aligned_aa = self._get_aligned_aa()

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def sequences(self):
        '''
        Sequences in the alignment.

        @return: {List} of {Sequence}
        '''
        return self._seq

    @property
    def identities(self):
        '''
        Identities in the alignment.

        @raises: {TypeError} if identities is _None_
        @return: {Integer}
        '''
        return int(self._identities)

    @property
    def precentage_identity(self):
        '''
        Percentage of identity.

        @raises: {TypeError} if identities is _None_
        @return: {Float}
        '''
        return 100 * float(self.identities) / self.aligned_aminoacids

    @property
    def positives(self):
        '''
        Positives in the alignment.

        @raises: {TypeError} if positives is _None_
        @return: {Integer}
        '''
        return int(self._positives)

    @property
    def precentage_positives(self):
        '''
        Percentage of positives (similarity).

        @raises: {TypeError} if positives is _None_
        @return: {Float}
        '''
        return 100 * float(self.positives) / self.aligned_aminoacids

    @property
    def gaps(self):
        '''
        Gaps in the alignment.

        @raises: {TypeError} if gaps is _None_
        @return: {Integer}
        '''
        return int(self._gaps)

    @property
    def alignment_pattern(self):
        '''
        Pattern of alignment. Summarizes the score of the alignment.

        @return: {String}
        '''
        return self._alipatt

    @property
    def aligned_aminoacids(self):
        '''
        Number of aligned amino acids.

        @return: {Integer}
        '''
        return self._aligned_aa

    @property
    def number_of_sequences(self):
        '''
        Number of sequences in the alignment.

        @return: {Integer}
        '''
        return self._num_seq

    ############
    # BOOLEANS #
    ############
    @property
    def is_multiple(self):
        '''
        Queries if we are dealing with a multiple alignment.

        @return: {Boolean}
        '''
        return self._num_seq > 2

    @property
    def are_segments_ok(self):
        '''
        Checks that the definition of the segments in the alignment is correct.

        @return: {Boolean}
        '''
        return self._segmentation_ok

    ###########
    # METHODS #
    ###########
    def add_complex_index(self, refseq, index, pregaps = 0):
        '''
        Manages the addition of a complex index to one of the sequences in the
        alignment.

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    index
        @pdef:     a complex index
        @ptype:    {List}

        @param:    pregaps
        @pdef:     a measure of the gap previous to the position
        @ptype:    {Integer}

        @raises: {AttributeError} if the index is not a list or if it
                 is a list smaller than the number of aligned residues in the
                 sequence.
        '''
        if not isinstance(index, (list, tuple)):
            raise AttributeError('A complex index is a list of IndexedNum or integers')
        if len(index) < len(re.sub('-', '', self._seq[refseq].sequence)):
            raise AttributeError('The given index is not long enough to cover the assigned sequence')

        data        = self._process_index(index)
        part_to_add = data[self._idx[refseq] - 1:]
        self._update_index(refseq, part_to_add)
        self._search_segments()

    def overlap(self, qrefseq, mappedrefseq, otherAli):
        '''
        Measures the overlap between the qrefseq of two alignments when both sequences
        are aligned over mappedrefseq.
        Overlap is measured over the shortest sequence and ranges between 0 and 1.

        mappedrefseq    ------------------------|-------|----------------------
        self.qrefseq     -----------------------|-------|
        otherAli.qrefseq                        |-------|--------------------
                                                 *******

        @param:    qrefseq
        @pdef:     reference sequence (remove lateral gaps from all the alignment).
                   those are the sequence which overlap we evaluate when mapped over
                   the mappedrefseq.
        @ptype:    {Integer}

        @param:    mappedrefseq
        @pdef:     sequence to evaluate overlap upon.
        @ptype:    {Integer}

        @param:    otherAli
        @pdef:     other alignment to which we are comparing. Is mandatory that
                   the alignment has at least as many sequences as the requested.
        @ptype:    {SeqAli}

        @return: {Float}
        '''
        length = min(len(self.sequences[mappedrefseq].ungapped()),
                     len(otherAli.sequences[mappedrefseq].ungapped()))
        mf = self._first_segment_value(qrefseq)
        ml = self._last_segment_value(qrefseq)
        me = self.get_section_from_sequence_position(qrefseq, mf, ml)

        hf = otherAli._first_segment_value(qrefseq)
        hl = otherAli._last_segment_value(qrefseq)
        he = otherAli.get_section_from_sequence_position(qrefseq, hf, hl)

        mf = me._first_segment_value(mappedrefseq)
        ml = me._last_segment_value(mappedrefseq)
        hf = he._first_segment_value(mappedrefseq)
        hl = he._last_segment_value(mappedrefseq)

        imax = self._max_position(mappedrefseq, mf, hl)
        emin = self._min_position(mappedrefseq, ml, hf)
        if (imax == mf and not mf == hl) or \
           (emin == ml and not ml == hf):
            return 0

        fmax = self._max_position(mappedrefseq, mf, hf)
        lmin = self._min_position(mappedrefseq, ml, hl)

        me = self.get_section_from_sequence_position(mappedrefseq, fmax, lmin)
        he = otherAli.get_section_from_sequence_position(mappedrefseq, fmax, lmin)

        l = self.get_sequence_section_from_sequence_position(mappedrefseq,
                                                             fmax, lmin,
                                                             gapped = False)

        return float(len(l)) / float(length)

    def get_sequence_section_from_sequence_position(self, refseq, ini, end, gapped = True):
        '''
        Retrieve a sequence of interest between two points of that sequence.

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    ini
        @pdef:     initial *sequence position*
        @ptype:    {Integer}

        @param:    end
        @pdef:     end *sequence position*
        @ptype:    {Integer}

        @param:    gapped
        @pdef:     decide whether to return or not the gaps in the sequence
        @pdefault: _True_
        @ptype:    {Boolean}

        @return: {Sequence}
        '''
        qseq = self.get_section_from_sequence_position(refseq, ini, end)._seq[refseq]

        if gapped:
            return qseq
        else:
            qseq.do_ungap()
            return qseq

    def get_segment_section_from_sequence_position(self, refseq, ini, end):
        '''
        Retrieves the section definition for a defined segment of the alignemnt.

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    ini
        @pdef:     initial *sequence position*
        @ptype:    {Integer}

        @param:    end
        @pdef:     end *sequence position*
        @ptype:    {Integer}

        @return: {List}
        '''
        return self.get_section_from_sequence_position(refseq, ini, end)._segment[refseq]

    def increment_sequence_index(self, refseq, new_index):
        '''
        If the start index is corrected _a posteriori_, (for example because the
        sequences are PDB sequences and they did not start in 1 in the first place)
        this correction needs to be extended to the section indexes.

        @param:    refseq
        @pdef:     identifier of the reference sequence
        @ptype:    {Integer}

        @param:    new_index
        @pdef:     new index by which to correct the rest.
        @ptype:    {Integer}
        '''
        self._idx[refseq] += (new_index - 1)
        for x in xrange(len(self._segment[refseq])):
            self._segment[refseq][x] = self._segment[refseq][x] - 1 + self._idx[refseq]

    def get_respective_coordinate(self, refseq, destseq, pos):
        '''
        From the *sequence position* of one sequence, obtain the *sequence position*
        of another.

        @param:    refseq
        @pdef:     identifier of the reference sequence
        @ptype:    {Integer}

        @param:    destseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    pos
        @pdef:     *sequence position* of the reference sequence
        @ptype:    {Integer}

        @return: {Integer}
        '''
        pos = self._alignment_position_from_sequence_position(refseq, pos)
        return self._sequence_position_from_alignment_position(destseq, pos)

    def get_coverage_of_sequence(self, refseq, refseq_full_length = None,
                                 ini = None, end = None):
        '''
        If refseq_full_length is specified, calculates the coverage of the alignment
        over the complete sequence (including the part that does not appear in
        the alignment)
        If ini and/or end are specified, it calculates how much that section of
        sequence is covered by the alignment. In multiple sequence alignments this
        implies that at least is aligned with one other sequence. They supersede
        refseq_full_length.

        @param:    refseq
        @pdef:     identifier of the reference sequence
        @ptype:    {Integer}

        @param:    refseq_full_length
        @pdef:     real length of the sequence, not only of the aligned region.
        @pclash:   ini, end. If specified any of them this is ignored
        @ptype:    {Integer}

        @param:    ini
        @pdef:     initial *sequence position* if we want to calculate only a section
        @ptype:    {Integer}

        @param:    end
        @pdef:     end *sequence position* if we want to calculate only a section
        @ptype:    {Integer}

        @return: {Float}
        '''
        is_section = (ini is not None or end is not None)

        ini = self._first_segment_value(refseq) if ini is None else ini
        end = self._last_segment_value(refseq) if end is None else end

        if ini is None and end is None:  # this sequence is all gaps
            return 0

        section = self.get_section_from_sequence_position(refseq, ini, end)

        if is_section:
            refseq_full_length = section._aligned_sequence_length(refseq)

        coverage = 0
        tk_seqs  = section._tokenize()
        for x in range(len(tk_seqs[refseq])):
            if bool(tk_seqs[refseq][x]) and sum(map(lambda n: int(n[x]), tk_seqs)) > 1:
                coverage += 1

        return float(coverage) / int(refseq_full_length)

    def get_section_from_sequence_position(self, refseq, ini, end):
        '''
        Extracts a section of the alignment according to the given positions
        of the sequence of interest.

        @param:    refseq
        @pdef:     identifier of the reference sequence
        @ptype:    {Integer}

        @param:    ini
        @pdef:     initial *sequence position*
        @ptype:    {Integer}

        @param:    end
        @pdef:     end *sequence position*
        @ptype:    {Integer}

        @return: {SeqAli}
        '''

        align_ini = self._alignment_position_from_sequence_position(refseq, ini)
        align_end = self._alignment_position_from_sequence_position(refseq, end)

        if align_ini > len(self) or align_end > len(self):
            raise IndexError
        if align_ini <= 0 or align_end <= 0:
            raise IndexError
        if align_ini == align_end:
            align_end = None if align_ini == len(self) else (align_end + 1)

        return self.__getitem__(slice(align_ini, align_end, 1))

    def add_alignment_pattern(self, align_pattern, method = 'blast'):
        '''
        Add a summary line that summarizes the alignment, as it exists in the
        blast output.
        If identities, positives and gaps have not been defined before, they
        are filled, as long as the summary format is known and its identification
        provided through the method attribute.

        @param:    align_pattern
        @pdef:     summary line of the alignment.
        @ptype:    {String}


        @param:    method
        @pdef:     format of the summary pattern
        @pdefault: 'blast'
        @ptype:    {String}
        '''
        self._alipatt   = align_pattern
        self._aliptmeth = method

        if self._positives is None and self._identities is None and self._gaps is None:
            if self._aliptmeth == 'blast' and not self.is_multiple:
                self._positives  = 0
                self._identities = 0
                self._gaps       = 0
                for x in range(len(self._alipatt)):
                    position = self._alipatt[x]
                    if position == '+':
                        self._positives  += 1
                    elif position == ' ':
                        if bool(re.search('-', "".join(self[x + 1]))):
                            self._gaps += 1
                    else:
                        self._identities += 1
                        self._positives  += 1
            else:
                return NotImplemented

    def trim_sequence_from(self, source_id, query_id, gapped = False):
        '''
        Extracts the part of the query sequence that aligns with the source
        sequence trimming the unaligned lateral sections.

        @param:    source_id
        @pdef:     identifier of the source sequence
        @ptype:    {Integer}

        @param:    query_id
        @pdef:     identifier of the query sequence that will be returned
        @ptype:    {Integer}

        @param:    gapped
        @pdef:     decide whether to return or not internal gaps in the sequence
        @pdefault: _False_
        @ptype:    {Boolean}

        @return: {String}
        '''
        seq = ''
        letter_found = False
        source = self.sequences[source_id]
        target = self.sequences[query_id]
        gap    = Sequence.GAP_DEFINITION
        for i in range(len(source)):
            if not letter_found:
                if not bool(re.search(gap, source[i])):
                    letter_found = True
                    if not bool(re.search(gap, target[i])) or gapped:
                        seq += target[i]
            else:
                if not bool(re.search(gap, source[i])):
                    if not bool(re.search(gap, target[i])) or gapped:
                        seq += target[i]
                else:
                    if len(re.sub(gap, '', source[i:])) > 0:
                        if not bool(re.search(gap, target[i])) or gapped:
                            seq += target[i]
                    else:
                        break
        return seq

    def evaluate_Rost_twilight_zone(self, equation = 'ID', parameter = None):
        '''
        Rost curves for evaluation homology.
        Use them to decide whether the aligned segments are homologous.

        Rost's Twilight Zone can only be applyed to alignments between
        TWO (2) SEQUENCES.
        This means is not allowed for MULTIPLE SEQUENCE ALIGNMENT.

        @param:    equation
        @pdef:     type of curve to evaluate homology
        @pdefault: 'identity'
        @poptions: 'id', 'identity', 'sim', 'similarity', 'hssp'
        @ptype:    {String}

        @@param:   parameter
        @pdef:     value of the corrector of the curve. The curve gets more
                   restrictive as the value gets bigger.
        @pdefault: 5 for 'identity', 12 for 'similarity' and 8 for 'hssp'
        @ptype:    {Integer}

        @raises: {SeqAliError} if it is multiple alignment.
        @raises: {AttributeError} if requested equation is not available.

        @return: {Boolean}
        '''
        if self.is_multiple:
            raise self._error.two_sequence_method()

        if equation.upper() in ['ID', 'IDENTITY'] :
            r = Rost.identity_curve(self.aligned_aminoacids, parameter)
            isHomolog = self.precentage_identity >= r
        elif equation.upper() in ['SIM', 'SIMILARITY']:
            r = Rost.similarity_curve(self.aligned_aminoacids, parameter)
            isHomolog = self.precentage_positives  >= r
        elif equation.upper() == 'HSSP':
            r = Rost.hssp_curve(self.aligned_aminoacids, parameter)
            isHomolog = self.precentage_identity >= r
        else:
            raise AttributeError('Selected equation not available.')

        return isHomolog

    def format_positions(self, human_readable = False):
        '''
        Creates a visual representation of the segments of the alignment.

        @param:    human_readable
        @pdef:     select if the visual representation is expected to be read
                   by humans.
        @pdefault: _False_
        @ptype:    {Boolean}

        @return: {String}
        '''
        if not human_readable:
            return self._compressed_segment_format()
        else:
            return self._human_segment_format()

    ###################
    # PRIVATE METHODS #
    ###################
    def _max_position(self, refseq, pos1, pos2):
        '''
        Gives whichever position is the biggest according to the sequence.

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    pos1
        @pdef:     position 1 to compare
        @ptype:    {IndexedNum}

        @param:    pos2
        @pdef:     position 2 to compare
        @ptype:    {IndexedNum}

        @return: {IndexedNum}
        '''
        p1, p2 = None, None
        try:
            p1 = self._alignment_position_from_sequence_position(refseq, pos1, static = True)
        except IndexError:
            pass
        try:
            p2 = self._alignment_position_from_sequence_position(refseq, pos2, static = True)
        except IndexError:
            pass
        
        if p1 is None:
            return pos2
        if p2 is None:
            return pos1

        if p1 >= p2:
            return pos1
        else:
            return pos2

    def _min_position(self, refseq, pos1, pos2):
        '''
        Gives whichever position is the smallest according to the sequence.

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    pos1
        @pdef:     position 1 to compare
        @ptype:    {IndexedNum}

        @param:    pos2
        @pdef:     position 2 to compare
        @ptype:    {IndexedNum}

        @return: {IndexedNum}
        '''
        maxim = self._max_position(refseq, pos1, pos2)
        if maxim == pos1:
            return pos2
        else:
            return pos1

    def _sequence_position_from_alignment_position(self, refseq, pos):
        '''
        Retrieve the position of a given sequence with respect to the *alignment
        position*. If the alignment position corresponds with a gap, it moves
        up to the next position.
        Remember that *alignment position* starts with 1 not 0.
        Inverse of _alignment_position_from_sequence_position

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    pos
        @pdef:     *alignment position* of interest
        @ptype:    {Integer}

        @return: {Integer}
        '''
        c = 0
        while self._seq2ali[refseq][pos - 1 + c] is None:
            c += 1
        return self._seq2ali[refseq][pos - 1 + c]

    def _alignment_position_from_sequence_position(self, refseq, pos, static = False):
        '''
        From a *sequence position* obtain the corresponding *alignment position*.
        Inverse of _sequence_position_from_alignment_position

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    pos
        @pdef:     *sequence position* of interest
        @ptype:    {Integer}

        @param:    static
        @pdef:     When _True_ calls the staticSA that contains the seq2ali before splits
        @pdefault: _False_
        @ptype:    {Boolean}

        @raise: {IndexError} if position is outside of aligned sequence
        @return: {Integer}
        '''
        if not static:
            data = self._seq2ali[refseq]
        else:
            data = self._staticSA[refseq]
        pos = IndexedNum(pos)
        try:
            return data.index(pos) + 1
        except ValueError:
            raise IndexError

    def _sequence_position_id(self, refseq, pos):
        '''
        retrieves the direct position of a requested sequence according to its
        defined init. Considers as if there were no gaps.

        Almost exclusively used for _search_segments()

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    pos
        @pdef:     *sequence position* of interest
        @ptype:    {Integer}

        @return: {Integer}
        '''
        return self._seq2ali[refseq][pos]

    def _first_segment_value(self, refseq):
        '''
        Report the starting values of a given sequence.

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @return: {Integer}
        '''
        for i in range(len(self)):
            if self._seq2ali[refseq][i] is not None:
                return self._seq2ali[refseq][i]

    def _last_segment_value(self, refseq):
        '''
        Report the finishing values of a given sequence.

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @return: {Integer}
        '''
        for i in reversed(range(len(self))):
            if self._seq2ali[refseq][i] is not None:
                return self._seq2ali[refseq][i]

    def _compressed_segment_format(self):
        '''
        Creates a compressed format to represent the segments of the alignment.
        The format is such as:
        qpos[0]:hpos[0],qpos[1]:hpos[1];qpos[2]:hpos[2],qpos[3]:hpos[3]; [...]
        ^       ^       ^       ^       ^       ^       ^       ^
        |       |       |       |       |       |       |       |Hit segment 2 end
        |       |       |       |       |       |       |Query segment 2 end
        |       |       |       |       |       |Hit segment 2 start
        |       |       |       |       |Query segment 2 start
        |       |       |       |Hit segment 1 end
        |       |       |Query segment 1 end
        |       |Hit segment 1 start
        |Query segment 1 start

        @return: {String}
        '''
        outdata = []
        for x in range(len(self._segment[0])):
            text = []
            for i in range(len(self._segment)):
                text.append("{0}".format(self._segment[i][x]))
            outdata.append(":".join(text))
            if x % 2:
                if x != len(self._segment[0]) - 1:
                    outdata.append(";")
            else:
                outdata.append(",")
        return "".join(outdata)

    def _human_segment_format(self):
        '''
        Creates a easy human readable format to represent the segments of the
        alignment.

        @return: {String}
        '''
        outdata = []
        for i in range(len(self._segment)):
            text = []
            for x in range(len(self._segment[i])):
                text.append("{0:>4}".format(self._segment[i][x]))
                if not x % 2:
                    text.append(" :")
                elif x != len(self._segment[0]) - 1:
                    text.append(" |")
            text = "".join(text)
            outdata.append(text)
        return "\n".join(outdata)

    def _tokenize(self):
        '''
        creates a binary representation of the alignment in an array.
        0 represents gaps while 1 represent aligned amino acids.

        @return: {List} of {String}
        '''
        token_seqs = []
        for seq in self._seq:
            token_seqs.append(seq.tokenize('binary'))
        return token_seqs

    def _get_aligned_aa(self):
        '''
        Determines the total number of aligned amino acids.
        At least two sequences need to be aligned to consider the amino acids
        aligned.

        @return: {Integer}
        '''
        this_len = 0
        tk_seqs = self._tokenize()

        for x in range(len(self._seq[0])):
            profile_str = "".join(map(lambda n: n[x], tk_seqs))
            profile_vle = sum([int(i) for i in profile_str])
            if profile_vle > 1:
                this_len += 1
        return this_len

    def _aligned_sequence_length(self, refseq):
        '''
        Calculates the lenght of a specified sequence of the alignment,
        that means without gaps.

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @return: {Integer}
        '''
        return len(re.sub('-', '', self.sequences[refseq].sequence))

    def _check_segments(self):
        '''
        Checks that the number of segment separators make sense and, thus,
        no problem has been found when analyzing the segments.

        @return: {Boolean}
        '''
        static_length = len(self._segment[0])
        if static_length % 2 != 0:
            return False
        for i in range(len(self._segment)):
            if len(self._segment[i]) != static_length:
                return False
        return True

    def _search_segments(self):
        '''
        Revises the alignment and separates the different aligned blocks.

        @return: {Boolean}
        '''
        previous_profile = ''
        self._segment = []
        for i in range(self._num_seq):
            previous_profile += '0'
            self._segment.append([])
        tk_seqs = self._tokenize()
        for x in range(len(self)):
            profile_str = "".join(map(lambda n: n[x], tk_seqs))

            for i in range(self._num_seq):
                p0 = self._sequence_position_id(i, x)
                p1 = self._sequence_position_id(i, (x - 1))
                b0 = bool(int(profile_str[i]))
                b1 = bool(int(previous_profile[i]))
                if x == 0:
                    self._segment[i].append(p0 if b0 else '-')
                else:
                    if profile_str != previous_profile:
                        self._segment[i].append(p1 if b1 else '-')
                        self._segment[i].append(p0 if b0 else '-')
                    if x == (len(self) - 1):
                        self._segment[i].append(p0 if b0 else '-')

            previous_profile = profile_str

        x = 0
        while (x < len(self._segment[0])):
            counter = 0
            for y in range(self._num_seq):
                counter = counter + (1 if self._segment[y][x] != '-' else 0)
            if counter == 1:
                for y in range(self._num_seq):
                    self._segment[y].pop(x)
                    self._segment[y].pop(x)
                x = x - 2
            x += 2
        if len(self) == 1:
            for y in range(self._num_seq):
                self._segment[y].append(self._segment[y][0])
        return self._check_segments()

    def _build_slice_alignment(self, sequences, index):
        '''
        This function helps the slicing of the alignment through slice objects as
        an iterable. It is required to support inheritance of the slicing.
        It covers special requirements of child objects when building themselves.

        @param:    sequences
        @pdef:     array of sequences for the new alignment
        @ptype:    {List}

        @param:    index
        @pdef:     transformation and adaptation of the slice object
        @ptype:    {List}

        @return: {IndexedSeqAli}
        '''
        seqInits  = ['' for x in range(self.number_of_sequences)]
        complex_indexes = {}
        for x in range(self.number_of_sequences):
            seqInits[x] = self._sequence_position_from_alignment_position(x, index[0])
            complex_indexes[x] = list(filter(None, self._seq2ali[x]))
            complex_indexes[x] = complex_indexes[x][complex_indexes[x].index(seqInits[x]):]
        newali = self.__class__(sequences, seqInits)
        for refident in complex_indexes:
            newali._update_index(refident, complex_indexes[refident])
            newali._search_segments()
        if not self._alipatt is None:
            newali.add_alignment_pattern(self._alipatt[index[0]-1:index[1]-1:index[2]], self._aliptmeth)
        return newali


    def _process_index(self, index):
        '''
        Enumerates the 'X' of the index

        @param:    index
        @pdef:     transformation and adaptation of the slice object
        @ptype:    {List}

        @return: {List}
        '''
        a = []
        x = 1
        for i in range(len(index)):
            add = index[i] if index[i] != 'X' else 'X' + str(x)
            a.append(IndexedNum(add))
            x   = x + 1 if index[i] == 'X' else x
        return a

    def _update_index(self, refseq, index):
        '''
        Updates the seq2ali list that allows moving from *sequence* to *alignment*
        position and back.

        @param:    refseq
        @pdef:     identifier of the sequence of interest
        @ptype:    {Integer}

        @param:    index
        @pdef:     transformation and adaptation of the slice object
        @ptype:    {List}

        '''
        a = list(self._seq2ali[refseq])
        c = 0
        x = 1
        for i in range(len(a)):
            if a[i] is not None:
                add  = index[c] if index[c] != 'X' else 'X' + str(x)
                a[i] = IndexedNum(add)
                x    = x + 1 if index[c] == 'X' else x
                c   += 1
        self._seq2ali[refseq] = tuple(a)

        self._idx[refseq] = self._first_segment_value(refseq)

    def _built_seq2ali(self, sequence_inits):
        '''
        builts the seq2ali vector required to move from *sequence* to *alignemnt*
        positions.

        @param:    sequence_inits
        @pdef:     initial number of each sequence of the alignment. (not all
                   sequences start alignment at 1)
        @ptype:    {List} of {Integer}

        @return: {List} of {Tuples}
        '''
        r = []
        for i in range(self.number_of_sequences):
            c, a = 0, []
            for j in range(len(self._seq[i])):
                s = self._seq[i][j]
                if s == '-':
                    a.append(None)
                else:
                    a.append(sequence_inits[i] + c)
                    c += 1
            r.append(tuple(a))
        return r

    #################
    # CLASS METHODS #
    #################
    def __len__(self):
        return len(self._seq[0])

    def __getitem__(self, key):
        # We correct the count as one would try to ask for the first position
        # as 1, not as 0...
        # Thus, we must consider that alignments start always in 1 (as global,
        # not for each particular sequence)
        try:
            int(key)
        except:
            if not isinstance(key, slice):
                raise TypeError()
        else:
            if int(key) > len(self):
                raise IndexError
            return map(lambda n: n[int(key) - 1], self._seq)

        index = key.indices(len(self) + 1)

        if key.stop is not None:
            #Remember! we go according to align position, not array!!
            index = (index[0], index[1] + 1, index[2])
        else:
            index = (index[0], index[1], index[2])
        sequences = ['' for x in range(self.number_of_sequences)]
        for i in range(*index):
            data = self[i]
            for x in range(len(data)):
                sequences[x] += data[x]
            if i == len(self):
                break
        ali = self._build_slice_alignment(sequences, index)
        ali._staticSA = self._staticSA
        return ali

    def __iter__(self):
        for x in range(1, len(self) + 1):
            yield self[x]

    def __repr__(self):
        data = []
        for x in range(self._num_seq):
            data.append('{0}\t{1}\t{2}'.format(self._first_segment_value(
                x), self._seq[x].sequence, self._last_segment_value(x)))
        if not self._alipatt is None:
            data.append('\t{0}'.format(self._alipatt))
        if not self._identities is None and not self._positives is None and not self._gaps is None:
            data.append(
                'I: {0._identities:<4} P: {0._positives:<4} G: {0._gaps:<4}'.format(self))
        data.append(self.format_positions(True))
        return "\n".join(data)


class Rost(object):
    '''
    FORMULA DEFINITIONS FOR ROST TWILIGHT ZONE:

    Rost, B. (1999). Twilight zone of protein sequence alignments.
    Protein Engineering, 12(2), 85 94.

    MODIFIED FROM J.Planas-Iglesias IMPLEMENTATION

    '''
    E = decimal.Decimal(repr(math.e))
    P = {'identity':   decimal.Decimal('-0.32'),
         'similarity': decimal.Decimal('-0.335'),
         'hssp':       decimal.Decimal('-0.562')}
    V = {'identity': 480,  'similarity': 420, 'hssp': 290.15}
    D = {'identity': 1000, 'similarity': 2000}
    N = {'identity':   decimal.Decimal(5),
         'similarity': decimal.Decimal(12),
         'hssp':       decimal.Decimal(8)}

    CALLS = frozenset(['ID', 'IDENTITY', 'SIMILARITY', 'SIM', 'HSSP'])

    @staticmethod
    def available_call(call):
        '''
        Checks if the way to refer to Rost curves is correct.

        @param:    call
        @pdef:     identifier of the curve of interest
        @ptype:    {String}

        @return: {Boolean}
        '''
        return call in Rost.CALLS

    @staticmethod
    def hssp_curve(aligned_aminoacids, curve_parameter = None):
        '''
        HSSP identity curve. As defined in Rost's article.
        Fulfilling the curve implies that the percentage of identity in the
        alignment is greater or equal than the returned value of the curve for
        a given number of aligned residues.
        According to the article, Rost's curves are more accurate, at the
        expense of coverage.
        Original description of the curve in:

        Sander, C., & Schneider, R. (1991). Database of homology-derived protein
        structures and the structural meaning of sequence alignment.
        Proteins, 9(1), 56-68.

        @param:    aligned_aminoacids
        @pdef:     total number of aligned amino acids.
        @ptype:    {Integer}

        @param:    curve_parameter
        @pdef:     (N). This parameters alters the high of the curve.
                   The bigger the number, more restrictive the curve is.
        @pdefault: 8. As defined in Rost's article.
        @ptype:    {Integer}

        @return: {Decimal}
        '''
        if curve_parameter is None:
            curve_parameter = Rost.N['hssp']

        if aligned_aminoacids >= 80:
            p = 25
        else:
            p = decimal.Decimal(repr(Rost.V['hssp']))
            p = p * pow(aligned_aminoacids, Rost.P['hssp'])

        return curve_parameter + p

    @staticmethod
    def identity_curve(aligned_aminoacids, curve_parameter = None):
        '''
        Rost identity curve.
        Fulfilling the curve implies that the percentage of identity in the
        alignment is greater or equal than the returned value of the curve for
        a given number of aligned residues.

        @param:    aligned_aminoacids
        @pdef:     total number of aligned amino acids.
        @ptype:    {Integer}

        @param:    curve_parameter
        @pdef:     (N). This parameters alters the high of the curve.
                   The bigger the number, more restrictive the curve is.
        @pdefault: 5. As defined in Rost's article.
        @ptype:    {Integer}

        @return: {Decimal}
        '''
        return Rost._curve(aligned_aminoacids, curve_parameter, 'identity')

    @staticmethod
    def similarity_curve(aligned_aminoacids, curve_parameter = None):
        '''
        Rost similarity curve.
        Fulfilling the curve implies that the percentage of similarity in the
        alignment is greater or equal than the returned value of the curve for
        a given number of aligned residues.

        @param:    aligned_aminoacids
        @pdef:     total number of aligned amino acids.
        @ptype:    {Integer}

        @param:    curve_parameter
        @pdef:     (N). This parameters alters the high of the curve.
                   The bigger the number, more restrictive the curve is.
        @pdefault: 12. As defined in Rost's article.
        @ptype:    {Integer}

        @return: {Decimal}
        '''
        return Rost._curve(aligned_aminoacids, curve_parameter, 'similarity')

    @staticmethod
    def _curve(aligned_aminoacids, curve_parameter, curve):
        '''
        Executes the requested curve.

        @param:    aligned_aminoacids
        @pdef:     total number of aligned amino acids.
        @ptype:    {Integer}

        @param:    curve_parameter
        @pdef:     (N). This parameters alters the high of the curve.
                   The bigger the number, more restrictive the curve is.
        @ptype:    {Integer}

        @param:    curve
        @pdef:     Which curve is requested.
        @poptions: 'similarity' or 'identity'
        @ptype:    {String}

        formula : N + (V * pow(L, P * (1 + pow(E, (-L / D)))))

        @return: {Decimal}
        '''
        if curve_parameter is None:
            curve_parameter = Rost.N[curve]
        else:
            curve_parameter = decimal.Decimal(curve_parameter)
        m = decimal.Decimal(repr(float(-aligned_aminoacids) / Rost.D[curve]))
        e = pow(Rost.E, m)
        k = Rost.P[curve] * (1 + e)
        curve_shape = Rost.V[curve] * pow(aligned_aminoacids, k)
        return curve_parameter + curve_shape


class SeqAliError(Exception):
    '''
    Manages different errors that can occur during the parsing of an alignment.

    '''

    _MSG = ''

    def __init__(self):
        pass

    def wrong_segmentation(self):
        self._MSG  = 'There has been a problem when parsing the different segments '
        self._MSG += 'of the alignment.'
        return self

    def two_sequence_method(self):
        self._MSG  = 'The required function can only be applied to two sequence'
        self._MSG += ' alignments.'
        return self

    def __str__(self):
        return SeqAliError._MSG



import re

def readFASTA(file):
    """
    It reads FASTA files and returns them as a dictionary
    Works both for one-line and multi-line FASTA files
    """
    regex_name = re.compile("^>(\S+)")
    name = ""
    Protein_dic = {}
    
    file_fd = open(file,'r')
    for line in file_fd:
        line = line.strip()
        n = regex_name.match(line)
        if n:
            name = n.group(1)
            Protein_dic[name]=""
        else:
            Protein_dic[name] += line
            
    return Protein_dic

def printFASTA(dictionary, multiple=True, file=""):
    """
    Prints a fasta dictionary into a file
    if multiple is True it will print a single file (with a name specified in file)
    if multiple is False it will print a different file for each sequence,
    being the file name the sequence ID with .fa extension
    """
    if multiple:
        file_fd = open(file,'w')
        for prot in dictionary:
            file_fd.write(">" + prot + "\n" + dictionary[prot] + "\n")
        file_fd.close()
    else:
        for prot in dictionary:
            file_fd = open(prot+".fa",'w')
            file_fd.write(">" + prot + "\n" + dictionary[prot] + "\n")
            file_fd.close()
    


import sys
import argparse
import os
import re
from Bio import SeqIO as SeqIE
from Bio import ExPASy
from Bio.Seq import Seq
from Bio import SeqRecord
from Bio.Alphabet import IUPAC
import ConfigParser
import itertools
import shutil
import subprocess
import time
import random
import numpy as np
import scipy.stats as st
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


# Add '.' to sys.path
src_path = os.path.abspath(os.path.dirname(__file__))
sys.path.append(src_path)
script_path = src_path+"/../scripts"
sys.path.append(script_path)

# Read configuration file
config = ConfigParser.ConfigParser()
config_file = os.path.join(script_path, 'config.ini')
config.read(config_file)

# Add SBI library path to sys.path
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','sbi_library_path'))
sys.path.append(src_path)

# Add functions path to sys.path    
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','functions_path'))
sys.path.append(src_path)

# Import functions
import functions
from SeqIO import *
from functions import *
from SBI.external.blast import blast_parser
from SBI.structure.contacts import Complex
from SBI.sequence import Sequence
from SBI.structure import PDB

def fileExist (file):
 return os.path.exists(file) and os.path.isfile(file)

def sum_of_tuples(a,b):
    x=[]
    size=max(len(a),len(b))
    for i in xrange(size):
        if i <  len(a) and i <  len(b): x.append(a[i]+b[i])
        if i >= len(a) and i <  len(b): x.append(b[i])
        if i <  len(a) and i >= len(b): x.append(a[i])
    return tuple(x)

def plotROC(a,out,title):
  p=[]
  q=[]
  r=[]
  fo=open(out+".dat","w")
  fo.write("#%9s\t%10s\t%10s\n"%("FPR","TPR","TNR"))
  for fpr in sorted(a.iterkeys()):
      p.append(fpr)
      tpr,tnr=a.get(fpr)
      fo.write("%10.5f\t%10.5f\t%10.5f\n"%(fpr,tpr,tnr))
      q.append(tpr)
      r.append(tnr)
  fo.close()
  x = np.array(p)
  y = np.array(q)
  z = np.array(r)
  plt.plot(x,y, ms=5, lw=2, alpha=0.7, mfc='cyan',label="TPR")
  plt.plot(x,z, ms=5, lw=2, alpha=0.7, mfc='firebrick',label="Specificity")
  plt.xlabel('false positive ratio')
  plt.ylabel('true positive ratio')
  plt.title("%s"%title)
  plt.grid(True)
  p=[]
  q=[]
  p.append(0.0)
  q.append(1.0)
  p.append(1.0)
  q.append(0.0)
  x = np.array(p)
  y = np.array(q)
  plt.plot(x,y, ms=5, lw=2, alpha=0.7, mfc='green',label="random")
  plt.legend(bbox_to_anchor=(0.85, 0.9), loc='upper left', borderaxespad=0.)
  plt.savefig(out)
  plt.close()

def plotSuccess(a,out,title):
  p=[]
  t=[]
  r=[]
  w=[]
  fo=open(out+".dat","w")
  fo.write("#%9s\t%10s\t%10s\t%10s\n"%("correlation","coverage","success","failure"))
  for correlation in sorted([float(c) for c in a.iterkeys() if c!= "neutral"]):
      p.append(float(correlation))
      coverage, success_ratio,fail_ratio = a.get(correlation)
      fo.write("%10s\t%10.5f\t%10.5f\t%10.5f\n"%(str(correlation),coverage,success_ratio,fail_ratio))
      t.append(coverage)
      r.append(success_ratio)
      w.append(fail_ratio)
  fo.close()
  x = np.array(p)
  y = np.array(t)
  z = np.array(r)
  q = np.array(w)
  plt.plot(x,y, ms=5, lw=2, alpha=0.7, mfc='cyan',label="coverage")
  plt.plot(x,z, ms=5, lw=2, alpha=0.7, mfc='firebrick',label="success")
  plt.legend(bbox_to_anchor=(0.15, 0.95), loc='upper left', borderaxespad=0.)
  plt.xlabel('Pearson correlation')
  plt.ylabel('ratio')
  plt.title("%s"%title)
  plt.grid(True)
  plt.savefig(out)
  plt.close()


def parse_user_arguments(*args, **kwds):
    parser = argparse.ArgumentParser(
        description = 'Prediction of gain/loss of interaction for  WT versus mutant forms using clusters of protein-protein interactions',
        epilog      = '@Oliva\'s lab 2018')
    parser.add_argument('-d', '--folder', dest = 'folder', action = 'store',
                        help = 'Input directory with MODPPI results')
    parser.add_argument('-g','--ddG',dest='ddg',action = 'store',default=None,
                        help = 'SKEMPI parsed file of affinities to check TPR and FPR (if empty no analysis is done)')
    parser.add_argument('-e', '--Energy_tested', dest = 'score', action = 'store',default="ddG_mean",
                        help = 'Energy to be used in the test')
    parser.add_argument('-i', '--input_ranks', dest = 'file_ranks', action = 'store',default=None,
                        help = 'Input file with the ranking conditions of the best correlation between predicted and real ddG')
    parser.add_argument('-o', '--output_directory', dest = 'outdir', action = 'store', default = 'selected_clusters',
                        help = 'Output directory (default is ModPPI_models)')
    parser.add_argument('-l','--label',dest='label',action = 'store',default='',
                        help = 'Label to identify the output "Compare_ddG_with_average_"')
    parser.add_argument('-k','--factor_lambda',dest='factor_lambda',action = 'store',default='1.0',type=float,
                        help = 'Factor factor_lambda to multiply the energies (default is 1) ')
    parser.add_argument('-x', '--cross_ddG', dest = 'cross', action = 'store_true',
                        help = 'Flag to produce all crossed pairs of ddG (default is False)')
    parser.add_argument('-v', '--verbose', dest = 'show', action = 'store_true',
                        help = 'Flag for verbose mode (default is False)')
    options = parser.parse_args()
    return options

def predictor(ddg_prediction,ddg,epsilon):
  if   ddg_prediction >  epsilon :    state_prediction="loss"
  if   ddg_prediction < -epsilon :    state_prediction="gain"
  if  abs(ddg_prediction) < epsilon : state_prediction="neutral"
  if ddg is not None:
     #      tp,fp,tn,fn
     loss = (0, 0, 0, 0)
     gain = (0, 0, 0, 0)
     same = (0, 0, 0, 0)
     if   ddg_prediction >  epsilon and ddg >  epsilon:      
          ok   = 1
          fail = 0
          #      tp,fp,tn,fn
          loss = (1, 0, 0, 0)
          gain = (0, 0, 1, 0)
          same = (0, 0, 1, 0)
     if   ddg_prediction < -epsilon and  ddg < -epsilon:      
          ok   = 1
          fail = 0
          #      tp,fp,tn,fn
          loss = (0, 0, 1, 0)
          gain = (1, 0, 0, 0)
          same = (0, 0, 1, 0)
     if  abs(ddg)<=epsilon and abs(ddg_prediction)<= epsilon: 
          ok   = 1
          fail = 0
          #      tp,fp,tn,fn
          loss = (0, 0, 1, 0)
          gain = (0, 0, 1, 0)
          same = (1, 0, 0, 0)
     if   ddg_prediction > epsilon and  ddg < epsilon:      
          ok   = 0
          fail = 1
          #      tp,fp,tn,fn
          loss = (0, 1, 0, 0)
          if ddg < -epsilon:
             gain = (0, 0, 0, 1)
          else:
             same = (0, 0, 0, 1)
     if   ddg_prediction < -epsilon and  ddg > -epsilon:      
          ok   = 0
          fail = 1
          #      tp,fp,tn,fn
          gain = (0, 1, 0, 0)
          if ddg > epsilon:
            loss = (0, 0, 0, 1)
          else:
            same = (0, 0, 0, 1)
     if  abs(ddg)>epsilon and abs(ddg_prediction)<= epsilon: 
          ok   = 0
          fail = 1
          #      tp,fp,tn,fn
          same = (0, 1, 0, 0)
          if ddg >  epsilon:
            loss = (0, 0, 0, 1)
          else:
            gain = (0, 0, 0, 1)

  return state_prediction,ok,fail, gain, loss, same

def test_equal(a,b):

    aa,ab=a.split("::")
    ba,bb=b.split("::")
    protein_A1=aa.split()[0]
    protein_A2=ab.split()[0]
    protein_B1=ba.split()[0]
    protein_B2=bb.split()[0]

    check=False

    if str(protein_A1) == str(protein_B1) and str(protein_A2) == str(protein_B2): check =True
    if str(protein_A1) == str(protein_B2) and str(protein_A2) == str(protein_B1): check =True
   
    return check
 

def tpr_fpr(tp,fp,tn,fn):
    tpr= fpr= tnr = 0.0
    if (tp+fp)>0: tpr = float(tp)/float(tp+fp)
    if (tn+fn)>0: fpr = float(fp)/float(tn+fn)
    if (tn+fn)>0: tnr = float(tn)/float(tn+fn)
    return tpr,tnr,fpr

def main():

 options=parse_user_arguments()

 files       = os.listdir(options.folder)
 score       = options.score
 outdir      = options.outdir
 input_dir   = options.folder
 input_rank  = options.file_ranks
 verbose     = options.show
 factor      = options.factor_lambda
 label_out   = options.label
 cross       = options.cross

 if not os.path.exists(outdir): os.makedirs(outdir)
 if not  fileExist(input_rank):
   sys.stderr.write("Missing File with ranking conditions\n")
   exit(0) 
 ddg_dict={}
 if options.ddg is not None:
   if verbose: sys.stdout.write("Parsing %s\n"%(options.ddg))
   if not fileExist(options.ddg):
    sys.stderr.write("File %s not found \n"%options.ddg)
   fa=open(options.ddg,"r")
   for line in fa:
    if line.startswith("#"): continue
    form,ddg=line.split()
    ddg_dict.setdefault(form,float(ddg))
   fa.close()


 ppi_sets=set()
 for input_file in files:
   if input_file.endswith(".ppi"): 
      ppi_sets.add(input_file)

 conditions={}
 fa=open(input_rank,"r")
 for line in fa:
   if line.startswith("#"): continue
   rank, pmi, pv , cluster, correlation, nmodels, sign, slope, y_axis = line.split()
   conditions.setdefault(float(correlation),[]).append((float(sign),float(pmi),float(pv),int(cluster),float(slope),float(y_axis)))
 fa.close()


 comparison_all={}
 for file_ppi in ppi_sets:
    ppis=[]
    fd=open(os.path.join(input_dir,file_ppi),'r')
    for line in fd:
      pair=line.split()
      ppis.append((pair[0],pair[1]))
    fd.close()
    root_ppi=file_ppi.rstrip(".ppi")
    data=root_ppi.split("_")
    label=data[0]
    wt=(data[1],data[-1])
    ddg_file=root_ppi+"."+score+"_distribution_statistic.out"
    if verbose: print "Check file  %s"%os.path.join(input_dir,ddg_file)
    if fileExist(os.path.join(input_dir,ddg_file)):
      if verbose: print "Open %s"%os.path.join(input_dir,ddg_file)
      fd=open(os.path.join(input_dir,ddg_file),'r')
      for line in fd:
       pair=line.strip().split("versus")
       statistic=line.strip().split()
       if len(statistic)>1:
         if statistic[-1] == "Statistic": statistic_type=statistic[0]
       if len(pair)>1:
        ppia=pair[0].split()[0]
        ppib=pair[1].split()[0]
        pval_averages=pair[1].split("p-value")[1]
        pvalue=pval_averages.split()[0]
        average_preform_a=pval_averages.split()[1]
        average_inform_a=average_preform_a.split("(")[1].split("#")[0]
        average_form_a="::".join(average_inform_a.split("-"))
        average_form_a_r=average_inform_a.split("-")[1]+"::"+average_inform_a.split("-")[0]
        average_value_a=pval_averages.split()[2]
        average_preform_b=pval_averages.split()[3]
        average_inform_b=average_preform_b.split("(")[1].split("#")[0]
        average_form_b="::".join(average_inform_b.split("-"))
        average_form_b_r=average_inform_b.split("-")[1]+"::"+average_inform_b.split("-")[0]
        average_value_b=pval_averages.split()[4]
        p,z=ppia.split("-")
        q,conformer_a= z.split("#")
        cluster_a,interface_a= conformer_a.split("_")
        overlap_a=int(interface_a.lstrip("i"))
        x,z=ppib.split("-")
        y,conformer_b = z.split("#")
        cluster_b,interface_b= conformer_b.split("_")
        overlap_b=int(interface_b.lstrip("i"))
        if int(cluster_a) == int(cluster_b):
          diff = float(average_value_b)-float(average_value_a)
          ddg_a = 0.0
          ddg_b = 0.0
          root_a=average_form_a.split("_")[0]
          mut_a ="_".join([mx for mx in average_form_a.split("_")[1:]])
          root_b=average_form_b.split("_")[0]
          mut_b ="_".join([mx for mx in average_form_b.split("_")[1:]])
          interactors_all=wt[0]+"::"+wt[1]
          if ddg_dict.has_key(average_form_a): ddg_a = ddg_dict.get(average_form_a)
          if ddg_dict.has_key(average_form_a_r): ddg_a = ddg_dict.get(average_form_a_r)
          if ddg_dict.has_key(average_form_b): ddg_b = ddg_dict.get(average_form_b)
          if ddg_dict.has_key(average_form_b_r): ddg_b = ddg_dict.get(average_form_b_r)
          if cross:
            if (ddg_dict.has_key(average_form_a) or ddg_dict.has_key(average_form_a_r)) and (ddg_dict.has_key(average_form_b) or ddg_dict.has_key(average_form_b_r)):
              if average_form_a == average_form_b or average_form_a_r == average_form_b or average_form_a == average_form_b_r or average_form_a_r == average_form_b_r:
                  ddg = None
              else:
                  ddg = ddg_b - ddg_a
              comparison_all.setdefault((interactors_all,statistic_type),[]).append((int(cluster_b),float(overlap_b),float(overlap_a),average_form_b,average_form_a,float(pvalue),float(average_value_b),float(average_value_a),diff,ddg))
          if (p,q) == wt or (q,p) == wt :
              diff=float(average_value_b)-float(average_value_a)
              if average_form_a == average_form_b or average_form_a_r == average_form_b or average_form_a == average_form_b_r or average_form_a_r == average_form_b_r:
                  ddg = None
              else:
                  if ddg_dict.has_key(average_form_b): ddg=ddg_dict.get(average_form_b)
                  if ddg_dict.has_key(average_form_b_r): ddg=ddg_dict.get(average_form_b_r)
              comparison_all.setdefault((interactors_all,statistic_type),[]).append((int(cluster_b),float(overlap_b),float(overlap_a),average_form_b,average_form_a,float(pvalue),float(average_value_b),float(average_value_a),diff,ddg))
          if (x,y) == wt or (y,x) == wt  :
              diff=float(average_value_a)-float(average_value_b)
              if average_form_a == average_form_b or average_form_a_r == average_form_b or average_form_a == average_form_b_r or average_form_a_r == average_form_b_r:
                  ddg = None
              else:
                  if ddg_dict.has_key(average_form_a): ddg=ddg_dict.get(average_form_a)
                  if ddg_dict.has_key(average_form_a_r): ddg=ddg_dict.get(average_form_a_r)
              comparison_all.setdefault((interactors_all,statistic_type),[]).append((int(cluster_b),float(overlap_a),float(overlap_b),average_form_a,average_form_b,float(pvalue),float(average_value_a),float(average_value_b),diff,ddg))
      fd.close()

 
 output =outdir+"/Predict_ddG_with_average_"+label_out+"_"+score+".dat"
 graph  =outdir+"/ROC_ddG_with_average_"+label_out+"_"+score
 outdata=outdir+"/ROC_ddG_with_average_"+label_out+"_"+score+".dat"

 epsilon= 0.5
 prediction={}
 tp_and_fp ={}
 okey  = 0
 wrong = 0
 #       tp,fp,tn,fn
 gain  = (0, 0, 0, 0)
 loss  = (0, 0, 0, 0)
 same  = (0, 0, 0, 0)
 for correlation in sorted(conditions.iterkeys(),reverse=True):
     if not conditions.has_key(correlation): continue
     parameters= conditions.get(correlation)
     step  = 0
     if verbose: sys.stdout.write("Checking correlation %s Gain= %s Loss= %s Neutral= %s\n"%(str(correlation),str(gain),str(loss),str(same)))
     for (sign,pmi,pv,cluster,slope,y_axis) in parameters:
       step = step + 1
       number_of_models = 0
       for (file_name,analysis),result in comparison_all.iteritems():
         if verbose and step==1 and number_of_models < 100: 
            sys.stdout.write("Checking Interaction %s with condition PMI %10.5f P-value %10.1e Cluster %d \n"%(file_name,float(pmi),float(pv),int(cluster)))
            number_of_models = number_of_models + 1
         if verbose and step==1 and number_of_models == 100: 
            sys.stdout.write("...continue with the rest of interactions\n")
            number_of_models = number_of_models + 1
         for check_cluster,overlap_b,overlap_a,form_b,form_a,pvalue,ave_b,ave_a,diff,ddg in sorted(result,key=lambda x: int(x[0]),reverse=True):
             if test_equal(form_b,form_a): continue
             if verbose and step==1 and number_of_models < 10:
                sys.stdout.write("\t-- Check features %s %s Cluster %s PMI_A %10s PMI_B %10s P-value %10s Predicted ddG: %10s Known ddG: %10s\n"%(form_a,form_b,str(check_cluster),str(overlap_a),str(overlap_b),str(pvalue),str(diff),str(ddg)))
             if prediction.has_key((form_a,form_b)) or prediction.has_key((form_b,form_a)): continue
             if (10*int(overlap_b) >= pmi or  len(form_b.split("_"))==1) and (10*int(overlap_a) >= pmi or len(form_a.split("_"))==1) and check_cluster<=cluster and pvalue < pv:
               ddg_prediction = factor*diff*slope + y_axis
               if verbose: sys.stdout.write("\t-- Found Interaction %s %s with condition PMI %10.5f P-value %10.1e Cluster %d with correlation %10s and significance %10.1e Gain= %s Loss= %s Neutral= %s\n"%(form_a,form_b,float(pmi),float(pv),int(cluster),str(correlation),float(sign),str(gain),str(loss),str(same)))
               if ddg is not None:
                 state_prediction, ok,fail, g, l, s = predictor(ddg_prediction,ddg,epsilon)
                 wrong= wrong + fail
                 okey = okey  + ok
                 gain=sum_of_tuples(gain,g)
                 loss=sum_of_tuples(loss,l)
                 same=sum_of_tuples(same,s)
               prediction.setdefault((form_b,form_a),(check_cluster,overlap_b,overlap_a,pvalue,ave_b,ave_a,correlation,sign,slope,y_axis,state_prediction,ddg_prediction,str(ddg),ok,fail,g,l,s))
     tp_and_fp.setdefault(correlation,(okey,wrong,gain,loss,same))

 size_data=float(okey+wrong)
 okey  = 0
 wrong = 0
 #       tp,fp,tn,fn
 gain  = (0, 0, 0, 0)
 loss  = (0, 0, 0, 0)
 same  = (0, 0, 0, 0)
 for (file_name,analysis),result in comparison_all.iteritems():
  for check_cluster,overlap_b,overlap_a,form_b,form_a,pvalue,ave_b,ave_a,diff,ddg in sorted(result,key=lambda x: int(x[0]),reverse=True):
    if prediction.has_key((form_a,form_b)) or prediction.has_key((form_b,form_a)): continue
    if test_equal(form_b,form_a): continue
    ddg_prediction = 0
    if ddg is not None:
      state_prediction, ok,fail, g, l, s = predictor(ddg_prediction,ddg,epsilon)
      wrong= wrong + fail
      okey = okey  + ok
      gain=sum_of_tuples(gain,g)
      loss=sum_of_tuples(loss,l)
      same=sum_of_tuples(same,s)
    prediction.setdefault((form_b,form_a),(check_cluster,overlap_b,overlap_a,pvalue,ave_b,ave_a,correlation,sign,slope,y_axis,state_prediction,ddg_prediction,str(ddg),ok,fail,g,l,s))
 tp_and_fp.setdefault("neutral",(okey,wrong,gain,loss,same))
               
 size_data_neutral=size_data+float(okey+wrong)

          
 fo=open(output,"w")
 fo.write("#%14s\t%15s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%20s\t%20s\t%20s\n"%("Protein_B","Protein_A","PMI_B","PMI_A","Pvalue","cluster","correlation","two-tail-prob","slope","Y-axis","dG_B","dG_A","Prediction","pred.ddG","ddG","success","fail","gain","loss","unaffected"))
 for pair,results in  prediction.iteritems():
     protein_b,protein_a = pair
     check_cluster,pmi_b,pmi_a,pvalue,dg_b,dg_a,correlation,sign,slope,y_axis,state_prediction,ddg_prediction,ddg,ok,f,g,l,s =  results
     fo.write("%15s\t%15s\t%10.5f\t%10.5f\t%10.1e\t%10d\t%10s\t%10.1e\t%10.5f\t%10.5f\t%10s\t%10s\t%10s\t%10.5f\t%10s\t%10d\t%10d\t%20s\t%20s\t%20s\n"%(protein_b,protein_a,pmi_b,pmi_a,pvalue,check_cluster,str(correlation),sign,slope,y_axis,dg_b,dg_a,state_prediction,ddg_prediction,ddg,ok,f,g,l,s))
 fo.close()


 if options.ddg is None: 
    if verbose: sys.stdout.write("Done!\n")
    exit(0)
 
 tpr_tnr_fpr_gain={}
 tpr_tnr_fpr_loss={}
 tpr_tnr_fpr_same={}
 coverage_success={}

 fo=open(outdata,"w")
 fo.write("#%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\t%10s\n"%("Correlation","Success","Fails","Succ.Ratio","Fail Ratio","Coverage","TPR_gain","TNR_gain","FPR_gain","TP_gain","FP_gain","TN_gain","FN_gain","TPR_loss","TNR_loss","FPR_loss","TP_loss","FP_loss","TN_loss","FN_loss","TPR_neutral","TNR_neutral","FPR_neutral","TP_neutral","FP_neutral","TN_neutral","FN_neutral"))
 for correlation in sorted(tp_and_fp.iterkeys()):
     correct,fails,gain,loss,same = tp_and_fp.get(correlation)
     if correct+fails > 0: success_ratio = correct/float(correct+fails)
     if correct+fails > 0: fail_ratio    = fails/float(correct+fails)
     if correlation=="neutral": 
        coverage = float(correct+fails)/size_data_neutral
     else:
        coverage = float(correct+fails)/size_data
     tp_gain,fp_gain,tn_gain,fn_gain  = gain
     tp_loss,fp_loss,tn_loss,fn_loss  = loss
     tp_same,fp_same,tn_same,fn_same  = same
     tpr_gain,tnr_gain,fpr_gain = tpr_fpr(tp_gain,fp_gain,tn_gain,fn_gain)
     tpr_loss,tnr_loss,fpr_loss = tpr_fpr(tp_loss,fp_loss,tn_loss,fn_loss)
     tpr_same,tnr_same,fpr_same = tpr_fpr(tp_same,fp_same,tn_same,fn_same)
     tpr_tnr_fpr_gain.setdefault(fpr_gain,(tpr_gain,tnr_gain))
     tpr_tnr_fpr_loss.setdefault(fpr_loss,(tpr_loss,tnr_loss))
     tpr_tnr_fpr_same.setdefault(fpr_same,(tpr_same,tnr_same))
     coverage_success.setdefault(correlation,(coverage,success_ratio,fail_ratio))
     fo.write("%10s\t%10d\t%10d\t%10.5f\t%10.5f\t%10.5f\t%10.5f\t%10.5f\t%10.5f\t%10d\t%10d\t%10d\t%10d\t%10.5f\t%10.5f\t%10.5f\t%10d\t%10d\t%10d\t%10d\t%10.5f\t%10.5f\t%10.5f\t%10d\t%10d\t%10d\t%10d\n"%(str(correlation),correct,fails,success_ratio,fail_ratio,coverage,tpr_gain,tnr_gain,fpr_gain,tp_gain,fp_gain,tn_gain,fn_gain,tpr_loss,tnr_loss,fpr_loss,tp_loss,fp_loss,tn_loss,fn_loss,tpr_same,tnr_same,fpr_same,tp_same,fp_same,tn_same,fn_same))
 fo.close()

 outroc=graph+"_gain_ROC.png"
 title="ROC gain interaction"
 plotROC(tpr_tnr_fpr_gain,outroc,title)

 outroc=graph+"_loss_ROC.png"
 title="ROC loss interaction"
 plotROC(tpr_tnr_fpr_loss,outroc,title)
 
 outroc=graph+"_neutral_ROC.png"
 title="ROC unaffected interaction"
 plotROC(tpr_tnr_fpr_same,outroc,title)
  
 outgraph=graph+"_success.png"
 title="Success,failure and coverage" 
 plotSuccess(coverage_success,outgraph,title)

  
 if verbose: sys.stdout.write("Done!\n")

# MAIN ###########################################################################################################

if __name__ == '__main__':
    main()   




from SBI.databases import TaxID

tables = {'taxid'  : 'taxid',
          'taxold' : 'taxid_old'}

class TaxID(TaxID):
    def __init__(self, taxid = None, inline = None):
        super(TaxID, self).__init__(taxid = taxid, inline = inline)

    def toSQL(self):
        if not self.has_old:
            return 'INSERT INTO {0} VALUES ({1.taxid},"{1.name}",{1.parent},"{1.rank}");'.format(tables['taxid'], self)
        else:
            if not self.has_new:
                return 'INSERT INTO {0} (oldid) VALUES ({1.taxid});'.format(tables['taxold'], self)
            else:
                return 'INSERT INTO {0} VALUES ({1.taxid},{1.new});'.format(tables['taxold'], self)


import math, os, sys, re
import BioLib.Tools.Dssp as Dssp
from BioLib.Tools.BioExceptions import *

class Structure(object):
    ''' 
    Structure class represents a protein chain within a PDB
    '''

    def __init__(self, pdb, chain):
        '''
        Constructor
        '''

        if chain == '': chain = 'A'

        self.id = pdb + "_" + chain
        self.pdb = pdb
        self.chain = chain
        self.residues = []
        self.loops = []
        self.uniprot_ref = None

    # GETTERS #

    def get_id(self):
        return self.id

    def get_pdb(self):
        return self.pdb

    def get_chain(self):
        return self.chain

    def get_uniprot_ref(self):
        return self.uniprot_ref

    def get_residues(self):
        return self.residues

    def get_residue_by_num(self, num):
        for residue in self.get_residues():
            if residue.get_num() == num:
                return residue
        sys.stderr.write('Residue number %d not found\n' % num)
        return None

    def get_loops(self):
        return self.loops

    def get_loop_by_id(self, subclass_id):
        for loop in self.get_loops():
            if loop.get_id() == subclass_id:
                return loop
        sys.stderr.write('Loop %s not found\n' % subclass_id)
        return None

    def get_number_residues(self):
        return len(self.get_residues())

    def get_number_atoms(self):
        atoms = 0
        for residue in self.get_residues():
            atoms += residue.get_number_atoms()
        return atoms            

    def get_first_residue(self):
        if self.get_number_residues() == 0:
            return None
        else:
            return self.get_residues()[0]

    def get_last_residue(self):
        if self.get_number_residues() == 0:
            return None
        else:
            return self.get_residues()[-1]

    # SETTERS #

    def set_uniprot_ref(self, uniprot_ref):
        self.uniprot_ref = uniprot_ref

    def set_residues(self, residues):
        self.residues = residues  
 
    def add_residue(self,residue):
        self.residues.append(residue)

    def set_loops(self, loops):
        for loop in loops:
            self.add_loop(loop)

    def add_loop(self, loop):
        loop_length = loop.get_end() - loop.get_start() + 1
        for i in xrange(loop_length):
            position = (loop.get_start()-1)+i+self.get_first_residue().get_num()
            if self.get_residue_by_num(position) != None:
                loop.add_residue(self.get_residue_by_num(position))
        loop.redefine_position((loop.get_start()-1)+self.get_first_residue().get_num(), (loop.get_end()-1)+self.get_first_residue().get_num())
        self.loops.append(loop)

    def set_dssp(self):
        '''
        Executes dssp and sets secondary structure (ss) and accesible surface area (acc) to each residue
        '''
        Dssp.execute_dssp(self)

    # CLASS METHODS #

    def clean(self):
        '''
        Cleans the structure removing residues without CA
        WARNING: DNA structures will empty
        NOTE: residues_tmp is needed to avoid remove items of the iterable during the iteration
        '''
        residues_tmp = [x for x in self.residues]
        for residue in residues_tmp:
            if not residue.has_ca():
                self.residues.remove(residue)

    def normalize_residues(self, start=1):
        '''
        Renumerate the residue numbers
        '''
        res_num = start
        for residue in self:
            residue.set_num(res_num)
            res_num += 1
        return res_num

    def normalize_atoms(self, start=1):
        '''
        Renumerate the atom numbers
        '''
        atom_num = start
        for residue in self:
            for atom in residue:
                atom.num = atom_num
                atom_num += 1
        return atom_num
        
    def get_sequence(self):
        '''
        Get the structure sequence
        '''
        seq = ''
        for residue in self:
            seq += residue.get_type_short()
        return seq 

    def get_self_interactions(self, c_type = 'CB', max_distance = 12, gap = 3, warnings = False):
        '''
        Compute the residue interactions intside the structure (folding interactions)
        '''
        interacting_residues = []
        used_residues = []
        max_distance = float(max_distance)
        for residue1 in self:
            used_residues.append(residue1)
            for residue2 in self:
                if residue2.get_num() not in [i for i in xrange(residue1.get_num()-gap, residue1.get_num()+gap+1)]:
                    try:
                        distance = residue1.get_residue_distance(residue2, c_type)
                    except ResidueDistanceError as e:
                        if warnings: sys.stderr.write(str(e))
                        continue
                    if distance <= max_distance and residue2 not in used_residues:
                        interacting_residues.append((residue1, residue2, distance))
        return interacting_residues

    def get_interacting_residues(self, structure, c_type = 'CB', max_distance = 12, uniq=True, warnings = False):
        '''
        Compute the interacting residues between two structure
        @Return List of interacting residues of both proteins. If unique is false, returns interacting
        pairs and their distances.
        '''
        max_distance = float(max_distance)
        interacting_residues_self = []
        interacting_residues_structure = []
        interacting_distances = []
        for residue1 in self:
            for residue2 in structure:
                try:
                    distance = residue1.get_residue_distance(residue2, c_type)
                except ResidueDistanceError as e:
                    if warnings: sys.stderr.write(str(e))
                    continue
                if distance <= max_distance:
                    interacting_residues_self.append(residue1)
                    interacting_residues_structure.append(residue2)
                    interacting_distances.append(distance)
        if uniq:
            return list(set(interacting_residues_self)), list(set(interacting_residues_structure))
        else:
            return interacting_residues_self, interacting_residues_structure, interacting_distances

    def get_opposite_interacting_residues(self, structure, size_sef = 1, size_struct = 1, c_type = 'CB', max_distance = 12, warnings=False):
        '''
        Returns a set of residues (for each structure) in the opposite faces of the interaction sites
        Used to compute back to back and partial interfaces
        '''
        interacting_residues = self.get_interacting_residues(structure, c_type=c_type, max_distance=max_distance, uniq=True, warnings=warnings)
        self_mass_center = self.__get_mass_center(interacting_residues[0])
        structure_mass_center = self.__get_mass_center(interacting_residues[1])
        self_residues_distances = []
        structure_residues_distances = []
        for residue in self:
            self_residues_distances.append((residue.get_ca().get_distance_coords(self_mass_center), residue))
        for residue in structure:
            structure_residues_distances.append((residue.get_ca().get_distance_coords(self_mass_center), residue))
        self_residues_distances.sort(reverse=True)
        structure_residues_distances.sort(reverse=True)
        self_residues_distances = [x[1] for x in self_residues_distances]
        structure_residues_distances = [x[1] for x in structure_residues_distances]
        return self_residues_distances[0:size_sef], structure_residues_distances[0:size_struct]

    def __get_mass_center(self, residues):
        '''
        Gets the mass center of a residue iterable (structure, list of residues...)
        '''
        x = 0
        y = 0
        z = 0
        for residue in residues:
            x += residue.get_ca().get_x()
            y += residue.get_ca().get_y()
            z += residue.get_ca().get_z()

        if len(residues) == 0:
            return None
        return x/len(residues), y/len(residues), z/len(residues)

    def residue_interacts(self, residue_num, structure, c_type = 'CB', max_distance = 12, warnings=False):
        '''
        Checks if a certain residue (specify by the number) interacts with other structure
        '''
        residue1 = self.get_residue_by_num(residue_num)
        interacts = False
        for residue2 in structure:
            try:
                distance = residue1.get_residue_distance(residue2, c_type)
            except ResidueDistanceError as e:
                if warnings: sys.stderr.write(str(e))
                continue
            if distance <= max_distance:
                interacts = True
                break
        return interacts

    def get_interacting_loops(self, structure, min_coverage = 0.8, c_type = 'CB', max_distance = 12, warnings=False):
        '''
        Get the interacting loops with other structure
        '''
        interacting_loops = []
        for loop in self.get_loops():
            coverage = loop.get_interacting_coverage(structure, c_type, max_distance, warnings)
            if min_coverage <= coverage:
                interacting_loops.append(loop)
        return interacting_loops

    def get_noninteracting_loops(self, structure, min_coverage = 0.8, c_type = 'CB', max_distance = 12, warnings=False):
        '''
        Get the non-interacting loops with other structure
        '''
        interacting_loops = self.get_interacting_loops(structure, min_coverage, c_type, max_distance, warnings)
        return list(set(self.get_loops()).difference(set(interacting_loops)))

    def relocate(self, translation_vector, rotation_vector, docking='HEX'):
        '''
        Relocate the structure position 
        @translation_vector = Vector used to translate the structure
        @Rotation_vector = Vector used to rotate the structure
        @docking = Docking program used to get the vectors
        '''
        for residue in self:
            try:
                residue.relocate(translation_vector, rotation_vector, docking)
            except RelocateProgramError as e:
                raise e

    def get_RMSD(self, structure):
        if self.get_number_residues() != structure.get_number_residues():
            sys.stderr.write("The RMSD needs to be between the same structures!!\n")
        E = 0
        for residue in xrange(self.get_number_residues()):
            for atom in xrange(self.get_residues()[residue].get_number_atoms()):
                distance = self.get_residues()[residue].get_atoms()[atom].get_distance(atom=structure.get_residues()[residue].get_atoms()[atom])
                E += math.pow(distance, 2)
        E = E /self.get_number_atoms()

        return math.sqrt(E)

    def get_radius(self):
        '''
        Get structure maximum radius (as defined in FTDock)
        '''
        largest = 0
        for residue in self:
            for atom in residue:
                present = math.pow(atom.get_x(), 2) + math.pow(atom.get_y(), 2) + math.pow(atom.get_z(), 2)
                if present > largest:
                    largest = present

        return math.sqrt(largest)

    def translate_onto_origin(self):
        '''
        Translate the structure to its center and return the translation vector
        '''
        atoms_num = 0
        x = 0
        y = 0
        z = 0

        for residue in self:
            for atom in residue:
                x = x + atom.get_x()
                y = y + atom.get_y()
                z = z + atom.get_z()
                atoms_num += 1

        x = x/atoms_num
        y = y/atoms_num
        z = z/atoms_num

        self.relocate((-x,-y,-z), (0,0,0))

    def get_N_linker(self):
        '''
        Get a list of N-terminal linker residues
        '''
        NLinker = []
        for residue in self:
            if residue.get_ss() == 'C':
                NLinker.append(residue)
            else:
               break
        return NLinker
 
    def get_N_linker_length(self):
        '''
        Get the length of the N-terminal linker
        '''
        return len(self.get_N_linker())

    def get_C_linker(self):
        '''
        Get a list of C-terminal linker residues
        '''
        CLinker = []
        for residue in self:
            if residue.get_ss() != 'C':
                CLinker = []
            else:
                CLinker.append(residue)
        return CLinker

    def get_C_linker_length(self):
        '''
        Get the length of the C-terminal linker
        '''
        return len(self.get_C_linker())

    def get_first_ss(self):
        '''
        Get the first residue with secondary structure
        '''
        firstResidue = None
        for residue in self:
            if residue.get_ss() != 'C':
                firstResidue = residue
                break
        return firstResidue

    def get_last_ss(self):
        '''
        Get the last residue with secondary structure
        '''
        lastResidue = None
        for residue in self:
            if residue.get_ss() != 'C':
                lastResidue = residue
        return lastResidue

    def __len__(self):
        '''
        Length of residues
        '''
        return len(self.residues)

    def __iter__(self):
        '''
        Iterates over each residue
        '''
        for residue in self.residues:
            yield residue

    def __str__(self):
        '''
        Returns the structure in PDB format
        '''
        structurePDB = ''
        for residue in self:
            for atom in residue:
                structurePDB += 'ATOM%s  %s%s%s%s%s%s%s\n' % (str(atom.get_num()).rjust(7),
                                                            atom.get_type().ljust(4),
                                                            residue.get_type().rjust(3),
                                                            self.get_chain().rjust(2),
                                                            str(residue.get_num()).rjust(4),
                                                            ('%.3f' %float(atom.get_x())).rjust(12),
                                                            ('%.3f' %float(atom.get_y())).rjust(8),
                                                            ('%.3f' %float(atom.get_z())).rjust(8))
        structurePDB += 'TER\n' 
        return structurePDB



import ConfigParser
import sys
import os 
from os import listdir
from os.path import isfile, join
import fnmatch

def main():
    
	print ("* ---- Building the config file ---- *") 


	script_dir = os.path.dirname(__file__)
	rel_path = os.path.join("scripts","config.ini")
	abs_file_path = os.path.join(script_dir, rel_path)
	config = ConfigParser.ConfigParser()
	config.read(abs_file_path)
	
	modppi_path =  os.path.dirname(os.path.abspath( __file__ ))
	python_path = sys.executable

	config.set('Paths', 'python_path', python_path)
	config.set('Paths', 'modppi_path', modppi_path)
	config.set('Paths', 'sbi_library_path', os.path.join(".", "src"))
	config.set('Paths', 'functions_path', os.path.join(".", "src", "functions" ))
	config.set('Paths', 'archdb_path',  os.path.join(modppi_path, "src"))
 
	config.set('Parameters','PPI_distance_threshold','8.0')
	config.set('Parameters','PPI_distance_threshold_shell','15.0')
	config.set('Parameters','PPI_threshold_type','cb')
	config.set('Parameters','overlap_interface','35.0')
	config.set('Parameters','min_ratio_common_poses','0.49')
	config.set('Parameters','n_parameter_twilight_zone','0')
	config.set('Parameters','extension_threshold','15')
	config.set('Parameters','hydrogens','full')
	config.set('Parameters','relax','yes')
	config.set('Parameters','ssp_score','ES3DC; ZES3DC; PAIR; ZPAIR; COMB; ZCOMB')
	config.set('Parameters','ssp_type','CB')



	with open(abs_file_path, 'w') as configfile:    
			config.write(configfile)	

	configuration = raw_input ("Press enter to continue with this configuration or enter \"n\" if you prefer to exit and edit config.ini file manually: ")
	if configuration== "n":
		print ("Please edit the config.ini file in the scripts directory of your Modpin installation folder.")
	else:	
                configuration_data = raw_input ("Press enter to configure your own storage of data \"n\" if you prefer to use the default configuration: ")

                if configuration_data =="n" or configuration_data =="N":
       			try:
          			if not os.path.isdir(os.path.join(modppi_path,"data")): os.makedirs(os.path.join(modppi_path,"data"))
          			if not os.path.isdir(os.path.join(modppi_path,"blast")):os.makedirs(os.path.join(modppi_path,"blast"))
        		except:
          			print "ERROR: You don't have permissions in this folder"
          			exit(0)
			config.set('Paths', 'data_path', os.path.join(".", "data"))
			config.set('Paths', 'blast_dir',  os.path.join(".", "blast"))
			config.set('Paths', 'pdb_path',  os.path.join(".", "data", "pdbsource"))
			config.set('Paths', '3did_path',  os.path.join(".", "data", "3did"))
			config.set('Files', 'fasta_list_file', os.path.join(".", "data", "nr.fa"))
			config.set('Files', 'nr90_list_file', os.path.join(".", "data", "merged_nr90.fa"))
			config.set('Files', 'ppi_files', os.path.join(".", "data", "pdb"))
			config.set('Files', 'ddi_files', os.path.join(".", "data", "3did"))
			config.set('Files', 'pdb_list_file', os.path.join(".", "data", "pdb.list"))
			config.set('Files', '3did_list_file', os.path.join(".", "data", "3did.list"))
			config.set('Files', 'database_file', os.path.join(".", "database.fasta"))
		else:
		 	data_path = raw_input("STRUCTURE) Insert the path to structural data files (should have pdbsource and 3did): ")
                	data_path = data_path.lstrip()
		 	while not find_file(data_path,"pdb*") or not  find_file(data_path,"3did"):
                    	  data_path = raw_input("Insert the correct path to structural data files (pdbsource or 3did are not found), or press enter to skip and add it later manually to the config.ini file: ")
                   	  data_path = data_path.lstrip()
                   	  if not data_path:
                   	     break;
                   	  else:
			     continue

                 	if find_file(data_path,"pdb*") and find_file(data_path,"3did"):
                    	  config.set('Paths', 'data_path',data_path)
		 	  config.set('Paths', 'pdb_path',  os.path.join(data_path, "pdbsource"))
		 	  config.set('Paths', '3did_path',  os.path.join(data_path, "3did"))
			  config.set('Files', 'ppi_files', os.path.join(data_path, "pdb"))
			  config.set('Files', 'ddi_files', os.path.join(data_path, "3did"))

		 	blast_dir = raw_input("SEQUENCES) Insert the path to sequence data formatted by BLAST/2.2.26: ")
			blast_dir = blast_dir.lstrip()
		 	while not os.path.isdir(blast_dir):
                    	  blast_dir = raw_input("Insert the correct path to sequence data files (folder not found), or press enter to skip and add it later manually to the config.ini file: ")
                   	  blast_dir = blast_dir.lstrip()
                   	  if not blast_dir:
                   	     break;
                 	  else:
			     continue

                	if os.path.isdir(blast_dir):
                 	    config.set('Paths', 'blast_dir',blast_dir)
			
                        database_fasta = "database.fasta"
                        while not find_file(blast_dir,database_fasta):
                          database_fasta = raw_input("Insert the name of the sequence database ('database.fasta' by default),  or press enter to skip and add it later manually to the config.ini file: ")
                          database_fasta = database_fasta.lstrip()
                          if not database_fasta:
                   	     break;
                 	  else:
			     continue

                 	if find_file(blast_dir,database_fasta):
			  config.set('Files', 'database_file', os.path.join(".", database_fasta))

                        fasta_list_file=os.path.join(data_path,"nr.fa")
		        while not os.path.isfile(fasta_list_file):
				fasta_list_file = raw_input("Insert the correct file of non-redundant sequences 'nr.fa', or press enter if it has to be created: ")
				fasta_list_file = fasta_list_file.lstrip()
				if not fasta_list_file: 
					break
				else:
					continue
                        if os.path.isfile(fasta_list_file):
				config.set('Files', 'fasta_list_file',fasta_list_file)

                        nr90_list_file=os.path.join(data_path,"merged_nr90.fa")
		        while not os.path.isfile(nr90_list_file):
				nr90_list_file = raw_input("Insert the correct file of non-redundant sequences merging PDB and 3DID 'merged_nr90.fa', or press enter if it has to be created: ")
				nr90_list_file = nr90_list_file.lstrip()
				if not nr90_list_file: 
					break
				else:
					continue
                        if os.path.isfile(nr90_list_file):
				config.set('Files', 'nr90_list_file',nr90_list_file)
                            
                        pdb_list_file=os.path.join(data_path,"pdb.list")
		        while not os.path.isfile(pdb_list_file):
				pdb_list_file = raw_input("Insert the correct file of list of PDB 'pdb.list', or press enter if it has to be created: ")
				pdb_list_file = pdb_list_file.lstrip()
				if not pdb_list_file: 
					break
				else:
					continue
                        if os.path.isfile(pdb_list_file):
				config.set('Files', 'pdb_list_file',pdb_list_file)


                        did_list_file=os.path.join(data_path,"3did.list")
		        while not os.path.isfile(did_list_file):
				did_list_file = raw_input("Insert the correct file of list of 3DID '3did.list', or press enter if it has to be created: ")
				did_list_file = did_list_file.lstrip()
				if not did_list_file: 
					break
				else:
					continue
                        if os.path.isfile(did_list_file):
				config.set('Files', '3did_list_file',did_list_file)
                            
                            



		modeller_path = raw_input("1) Insert path to bin folder of Modeller: ")
		modeller_path=modeller_path.lstrip()
		while not find_file(modeller_path, "mod*.*"):
			modeller_path = raw_input("Insert the correct path to bin folder of Modeller, or press enter to skip and add it later manually to the config.ini file: ")
			modeller_path = modeller_path.lstrip()
			if not modeller_path:
				break
			else:
				continue
		if find_file(modeller_path, "mod*.*"):
			config.set('Paths', 'modeller_path', modeller_path)

		blast_path = raw_input("2) Insert path to bin folder of BLAST/2.2.26: ")
		blast_path =  blast_path.lstrip()
		while which(os.path.join(blast_path, "blastpgp"))==False or os.path.isdir(blast_path)==False:
			blast_path = raw_input ("Insert the correct path to executable of BLAST, or press enter to skip and add it later manually to the config.ini file: ")	
			blast_path =  blast_path.lstrip()
			if not blast_path:
				break
			else:
				continue
		if which(os.path.join(blast_path, "blastpgp"))==True:
			config.set('Paths', 'blast_path', blast_path) 
	
		zrank_path = raw_input("3) Insert path to executable of ZRANK: ")
		zrank_path = zrank_path.lstrip()
		while os.path.isfile(zrank_path) == False:
			zrank_path  = raw_input ("Insert the correct path to executable of ZRANK, or press enter to skip and add it later manually to the config.ini file: ")
			zrank_path = zrank_path.lstrip()
			if not zrank_path :
				break
			else:
				continue
		if  os.path.isfile(zrank_path)==True:
			config.set('Paths', 'zrank_path', zrank_path)
	
		foldx_path = raw_input("4) Insert path to executable of FOLDX: ")
		foldx_path = foldx_path.lstrip()
		while os.path.isfile(foldx_path) == False:
			foldx_path  = raw_input ("Insert the correct path to executable of FoldX, or press enter to skip and add it later manually to the config.ini file: ")
			foldx_path = foldx_path.lstrip()
			if not foldx_path :
				break
			else:
				continue
		if  os.path.isfile(os.path.join(foldx_path,"foldx"))==True and os.path.isfile(os.path.join(foldx_path,"rotabase.txt"))==True:
			config.set('Paths', 'foldx_path', foldx_path)
                else:
                        sys.stdout.write("\t-- Warning!!! Neither 'foldx' nor 'rotabase.txt' were found under FoldX path\n")

		clustal_path = raw_input("5) Insert path to bin folder of clustalw2: ")
		clustal_path = clustal_path.lstrip()
		while which(os.path.join(clustal_path, "clustalw2")) == False or os.path.isdir(clustal_path) == False:
			clustal_path = raw_input ("Insert the correct path to executable of clustalw2, or press enter to skip and add it later manually to the config.ini file: ")
			clustal_path = clustal_path.lstrip()
			if not clustal_path:
				break
			else:
				continue
		if  which(os.path.join(clustal_path, "clustalw2")) == True:
			config.set('Paths', 'clustal_path', clustal_path)

		rosetta_path = raw_input("6) Insert path to the main folder Rosetta: ")
		rosetta_path = rosetta_path.lstrip()
		while os.path.isdir(os.path.join(rosetta_path, "database"))  == False:
			rosetta_path = raw_input ("Insert the correct path to Rosetta main folder which contains the database, or press enter to skip and add it later manually to the config.ini file: ")		
			rosetta_path = rosetta_path.lstrip()
			if not rosetta_path:
				break
			else:
				continue

		if  os.path.isdir(os.path.join(rosetta_path, "database"))  == True:
			config.set('Paths', 'rosetta_path', rosetta_path)

			relax_exe = os.path.join(rosetta_path, "source", "bin", "fixbb.linuxgccrelease")
                        if find_file( os.path.join(rosetta_path, "source", "bin"),"fixbb*"): 
                           sys.stdout.write("\t-- Found fixbb, default to be used for relaxing the structures: %s\n"%found_file(os.path.join(rosetta_path, "source", "bin"),"fixbb*"))
                           relax_exe = os.path.join(rosetta_path, "source", "bin", found_file(os.path.join(rosetta_path, "source", "bin"),"fixbb*"))
                        elif find_file( os.path.join(rosetta_path,"bin"),"fixbb*"):
                           sys.stdout.write("\t-- Found fixbb, default to be used for relaxing the structures: %s\n"%found_file(os.path.join(rosetta_path,"bin"),"fixbb*"))
                           relax_exe = os.path.join(rosetta_path, "bin", found_file(os.path.join(rosetta_path, "bin"),"fixbb*"))
                        elif find_file( os.path.join(rosetta_path,"source","bin"),"relax*"):
                           sys.stdout.write("\t-- Found relax, default to be used for relaxing the structures: %s\n"%found_file(os.path.join(rosetta_path, "source", "bin"),"relax*"))
                           relax_exe = os.path.join(rosetta_path, "source", "bin", found_file(os.path.join(rosetta_path, "source","bin"),"relax*"))
                        elif find_file( os.path.join(rosetta_path,"bin"),"relax*"):
                           sys.stdout.write("\t-- Found relax, default to be used for relaxing the structures: %s\n"%found_file(os.path.join(rosetta_path, "bin"),"relax*"))
                           relax_exe = os.path.join(rosetta_path, "bin", found_file(os.path.join(rosetta_path, "bin"),"relax*"))
			else:
                           sys.stdout.write("\t-- Neither FIXBB nor RELAX executables were found under ROSETTA path\n")
                              
			if os.path.isfile(relax_exe)== True:
				config.set('Paths', 'relax_exe', relax_exe)
			else: 
				while os.path.isfile(relax_exe)  == False:
					relax_exe = raw_input (" -Relaxing backbone-design application fixbb.linuxgccrelease (or relax.linuxgccrelease) not found under Rosetta bin folder. \n  Insert the correct path wih the name of executable, or press enter to skip and add it later manually to the config.ini file: ")
					if not relax_exe:
						break
					else:
						continue 
			if os.path.isfile(relax_exe)== True:
				config.set('Paths', 'relax_exe', relax_exe)


			interface_analyzer = os.path.join(rosetta_path, "source", "bin", "InterfaceAnalyzer.linuxgccrelease")
                        if find_file( os.path.join(rosetta_path, "source", "bin"),"InterfaceAnalyzer*"):
                          sys.stdout.write("\t-- Found InterfaceAnalyzer: %s\n"%found_file(os.path.join(rosetta_path, "source", "bin"),"InterfaceAnalyzer*"))
			  interface_analyzer = os.path.join(rosetta_path, "source", "bin", found_file(os.path.join(rosetta_path, "source", "bin"),"InterfaceAnalyzer*"))
                        elif find_file(os.path.join(rosetta_path, "bin"),"InterfaceAnalyzer*"):
                          sys.stdout.write("\t-- Found InterfaceAnalyzer: %s\n"%found_file(os.path.join(rosetta_path,  "bin"),"InterfaceAnalyzer*"))
			  interface_analyzer = os.path.join(rosetta_path, "bin", found_file(os.path.join(rosetta_path,  "bin"),"InterfaceAnalyzer*"))
			else:
                           sys.stdout.write("\t-- INTERFACEANALYZER executable was not found under ROSETTA path\n")

			if os.path.isfile(interface_analyzer)== True:
				config.set('Paths', 'interface_analyzer', interface_analyzer)
			else: 
				while os.path.isfile(interface_analyzer)  == False:
					interface_analyzer = raw_input(" -Interface-analyzer application InterfaceAnalyzer.linuxgccrelease not found under Rosetta bin folder. \n  Insert the correct path wih the name of executable, or press enter to skip and add it later manually to the config.ini file: ")
					if not interface_analyzer:
						break
					else:
						continue 
			if os.path.isfile(interface_analyzer)== True:
				config.set('Paths', 'interface_analyzer', interface_analyzer)

		

		hbplus_path = raw_input("7) Insert path to executable of hbplus: ")
		hbplus_path = hbplus_path.lstrip()
		while os.path.isfile(hbplus_path) == False:
			hbplus_path = raw_input ("Insert the correct path to executable of hbplus, or press enter to skip and add it later manually to the config.ini file: ")
			hbplus_path = hbplus_path.lstrip()
			if not hbplus_path:
				break
			else:
				continue
		if   os.path.isfile(hbplus_path)  == True:
			config.set('Paths', 'hbplus_path', hbplus_path)

		reduce_path = raw_input("8) Insert path to executable of reduce: ")
		reduce_path = reduce_path.lstrip()
		while os.path.isfile(reduce_path) == False:
			reduce_path = raw_input ("Insert the correct path to executable of reduce, or press enter to skip and add it later manually to the config.ini file: ")
			reduce_path = reduce_path.lstrip()
			if not reduce_path:
				break
			else:
				continue
		if  os.path.isfile(reduce_path)  == True:
			config.set('Paths', 'reduce_path', reduce_path)




		reduce_db_path = raw_input("9) Insert path and name of HET dict. file for reduce: ")
		reduce_db_path = reduce_db_path.lstrip()
		path, filename = os.path.split(reduce_db_path)
		while os.path.isfile(reduce_db_path)== False or filename.startswith("reduce")==False  or filename.endswith(".txt") == False:
			reduce_db_path = raw_input ("Insert the correct path and name of reduce dictionary, or press enter to skip and add it later manually to the config.ini file: ")
		        path, filename = os.path.split(reduce_db_path)
			if not reduce_db_path:
				break
			else:
				continue
		if os.path.isfile(reduce_db_path)==True and filename.startswith("reduce")==True and filename.endswith(".txt") == True:
				config.set('Paths', 'reduce_db_path', reduce_db_path)

                configure_cluster = raw_input ("10) Do you want configure the parallelization? (Y/N): ")
                if configure_cluster.startswith("y") or configure_cluster.startswith("Y"):
                   cluster_name = raw_input ("\t-Cluster name: ")
                   if not cluster_name: cluster_name="None"
                   cluster_queue = raw_input ("\t-Name of queue to be used in the cluster: ")
                   if not cluster_queue: cluster_queue="None"
                   cluster_submit = raw_input ("\t-Submission method; usually this is 'submit' or 'sbatch' (default is 'submit'): ")
                   if not cluster_submit: cluster_submit="submit"
                   cluster_qstat = raw_input ("\t-Command to show queues; usually this is 'qstat' or 'squeue' (default is 'qstat'): ")
                   if not cluster_qstat: cluster_qstat="qstat"
                   command_queue = raw_input ('\t-File with header commands (i.e. shell and environmental variables)\n\t required for running scripts in the cluster: (default is "command_queues.txt" empty file)')
	           if not command_queue: command_queue=os.path.join(".","command_queues.txt")
                   config.set('Cluster','cluster_name',cluster_name)
                   config.set('Cluster','cluster_queue',cluster_queue)
                   config.set('Cluster','cluster_submit',cluster_submit)
                   config.set('Cluster','cluster_qstat',cluster_qstat)
                   config.set('Cluster','max_jobs_in_queue','750')
                   config.set('Cluster','min_jobs_in_queue','5')
                   config.set('Cluster','command_queue', command_queue)


		with open(abs_file_path, 'w') as configfile:    
			config.write(configfile)

                print "\nDone"



def find_file(path, pattern):
			p =pattern
                        found=False
			if os.path.exists(path):
				for file in os.listdir(path):
                                        if found: break
					if fnmatch.fnmatch(file, p):
						found=True
			else:
				return False
                        return found

def found_file(path, pattern):
			p =pattern
                        found=False
			if os.path.exists(path):
				for file in os.listdir(path):
                                        if found: break
					if fnmatch.fnmatch(file, p):
						found=True
                                                found_file=file
			else:
				return False
                        return found_file
	
#check if exe path inserted by user exists
def which(program):
	fpath, fname = os.path.split(program)
	if fpath:
		if is_exe(program):
			return True
		else:
			for path in os.environ["PATH"].split(os.pathsep):
				exe_file = os.path.join(path, program)
				for elem in ext_elem(exe_file):
				  if is_exe(elem):
				    return True
	return False



def is_exe(fpath):
	return os.path.isfile(fpath) and os.access(fpath, os.X_OK) and os.path.isfile(fpath)	
def ext_elem(fpath):
 yield fpath
 for ext in os.environ.get("PATHEXT", "").split(os.pathsep):
	yield fpath + ext 

if __name__ == '__main__':
    main()




"""
    Support Library to calculate RMSD difference between two Structure Objects

                                        jbonet @ boliva's lab        2012
"""
import math
import numpy as np


def superimposition_matrix_vector(structure1, structure2):

    Matrix = kabsch(structure1, structure2)

    # structure1 = structure1.duplicate(hetero = False, water = False, backbone = True)
    # structure1.rotate(Matrix)

    Vector = structure1.geometric_center(structure = True, hetero = False, water = False) - \
        structure2.geometric_center(structure = True, hetero = False, water = False)

    for i in range(3):
        for j in range(3):
            if (Matrix[i][j] > 0 and Matrix[i][j] < 1e-5) or (Matrix[i][j] < 0 and Matrix[i][j] > -1e-5):
                Matrix[i][j] = 0
    return Matrix, Vector


def kabsch(str1 = None, str2 = None):
    '''
    Given two structures, kabsch algorithm tries to identify the rotation matrix to move str1 and optimize the RMSD over str2
    '''

    '''
    STEP1: Center into origin
    '''
    centered_str1 = str1.duplicate(hetero = False, water = False, backbone = True)
    centered_str2 = str2.duplicate(hetero = False, water = False, backbone = True)

    centered_str1.translate_onto_origin()
    centered_str2.translate_onto_origin()

    '''
    STEP2: Retrieve all coordinates
    '''
    centered_str1_coordinates = centered_str1.get_backbone_coordinates()
    centered_str2_coordinates = centered_str2.get_backbone_coordinates()

    V, S, Wt = np.linalg.svd(np.dot(np.transpose(centered_str2_coordinates), centered_str1_coordinates))

    reflect = float(str(float(np.linalg.det(V) * np.linalg.det(Wt))))
    if reflect == -1.0:
        S[-1] = -S[-1]
        V[:, -1] = -V[:, -1]

    U = np.dot(V, Wt)

    return U


# def repositioned_RMSD(str1 = None, str2 = None, c_type = "all", selection = None):
#     '''
#     Checking the RMSD between two IDENTICAL structures implies calculating the difference between all their atoms.
#     Structures are assumed to have been previously positioned as they are expected to be for the comparison

#     This definition accepts that some flexibility might have been applied to the structures
#     '''

#     '''
#     For lack of a better/faster way to check it out, we will consider that two structures are the same if they
#     have the same number of atoms.
#     Else, we will rise a ValueError
#     '''
#     if str1.get_structure_length() != str2.get_structure_length():
#         raise ValueError("Both structures need to be the same for the RMSD of the reposition to be calculated\n")

#     '''
#     Several flags can be called over this function:
#         c_type: the allowed values are:
#             all: it will compare the full RMSD
#             ca: it will compare the RMSD between the CA atoms
#             cb: it will compare the RMSD between the CB atoms -> GLY are ignored
#             cb-ca: CB RMSD - CA RMSD -> kind of compares only how the lateral chains have changed
#             any other option defaults to 'all'
#         selection: an array of positions can be submitted and only those Aa are going to be compared
#                    This can be used, for example, if comparing the Interface RMSD
#     '''
#     allowed_c_type = set(['all', 'ca', 'cb', 'cb-ca'])
#     if c_type not in allowed_c_type:
#         c_type = 'all'

#     if selection is not None:
#         selection_set = set(selection)

#     E = 0
#     atom_count = 0
#     for res in xrange(str1.get_structure_length()):
#         if selection is None or str1.get_residues()[res].get_residue_num() in selection_set:
#             if c_type == 'all':
#                 for at in xrange(str1.get_residues()[res].get_residue_size()):
#                     distance = str1.get_residues()[res].get_atoms()[at].get_distance(atom2=str2.get_residues()[res].get_atoms()[at])
#                     atom_count += 1
#                     E += math.pow(distance, 2)
#             if c_type == 'ca':
#                 distance = str1.get_residues()[res].get_ca().get_distance(atom2=str2.get_residues()[res].get_ca())
#                 atom_count += 1
#                 E += math.pow(distance, 2)
#             if c_type == 'cb' and str1.get_residues()[res].get_cb() is not None:
#                 distance = str1.get_residues()[res].get_cb().get_distance(atom2=str2.get_residues()[res].get_cb())
#                 atom_count += 1
#                 E += math.pow(distance, 2)
#             if c_type == 'cb-ca' and str1.get_residues()[res].get_cb() is not None:
#                 distance_a = str1.get_residues()[res].get_ca().get_distance(atom2=str2.get_residues()[res].get_ca())
#                 distance_b = str1.get_residues()[res].get_cb().get_distance(atom2=str2.get_residues()[res].get_cb())
#                 atom_count += 12
#                 E += math.pow(distance_b - distance_a, 2)

#     E = E / atom_count

#     return math.sqrt(math.fabs(E))


# def repositioned_RMSD_byMatrix(AV = None, AV2 = None, NormL = None, rotation_matrix1 = None, rotation_matrix2 = None,
#                                translation_vector1 = None, translation_vector2 = None):
#     '''
#     This method will try to implement an RMSD calculation based on the rotation matrices and the translation vectors
#     '''

#     x, y, z, V2 = calculateVandV2(translation_vector1 = translation_vector1, translation_vector2 = translation_vector2)
#     V  = [x, y, z]

#     V1 = calculateV1(rotation_matrix1 = rotation_matrix1, rotation_matrix2 = rotation_matrix2, AV2 = AV2)
#     V3 = calculateV3(V = V, rotation_matrix1 = rotation_matrix1, rotation_matrix2 = rotation_matrix2, AV = AV)

#     '''
#     Calculate RMSD
#     '''
#     rmsd = V2 + NormL - V1 + V3
#     return math.sqrt(math.fabs(rmsd))

# """
#     __Suporting Function to repositioned_RMSD_byMatrix__
# """


# def calculateVandV2(translation_vector1 = None, translation_vector2 = None):
#     '''
#     The translation vector clusters the data in order x, y, z
#     This one creates the vectors:
#         V:    x, y, z
#     And the value V2
#     '''
#     x = translation_vector1[0] - translation_vector2[0]
#     y = translation_vector1[1] - translation_vector2[1]
#     z = translation_vector1[2] - translation_vector2[2]

#     v2 = x * x + y * y + z * z

#     return x, y, z, v2


# def calculateV3(V = None, rotation_matrix1 = None, rotation_matrix2 = None, AV = None):
#     '''
#     This one calculates V3 from the arrays V and AV and the rotation matrices
#         AV[0] : average_x
#         AV[1] : average_y
#         AV[2] : average_z
#         V[0]  : x
#         V[1]  : y
#         V[2]  : z
#     '''

#     v3 = 0

#     v3 = v3 + ((rotation_matrix1[0][0] - rotation_matrix2[0][0]) * AV[0] +
#                (rotation_matrix1[0][1] - rotation_matrix2[0][1]) * AV[1] +
#                (rotation_matrix1[0][2] - rotation_matrix2[0][2]) * AV[2]) * V[0]
#     v3 = v3 + ((rotation_matrix1[1][0] - rotation_matrix2[1][0]) * AV[0] +
#                (rotation_matrix1[1][1] - rotation_matrix2[1][1]) * AV[1] +
#                (rotation_matrix1[1][2] - rotation_matrix2[1][2]) * AV[2]) * V[1]
#     v3 = v3 + ((rotation_matrix1[2][0] - rotation_matrix2[2][0]) * AV[0] +
#                (rotation_matrix1[2][1] - rotation_matrix2[2][1]) * AV[1] +
#                (rotation_matrix1[2][2] - rotation_matrix2[2][2]) * AV[2]) * V[2]

#     return 2 * v3


# def calculateV1(rotation_matrix1 = None, rotation_matrix2 = None, AV2 = None):
#     '''
#     This one calculates V1 from the rotation matrices and the AV2 vector
#     '''

#     v1 = 0

#     v1 = v1 + (rotation_matrix1[0][0] * rotation_matrix2[0][0] +
#                rotation_matrix1[1][0] * rotation_matrix2[1][0] +
#                rotation_matrix1[2][0] * rotation_matrix2[2][0]) * AV2[0]  # xx
#     v1 = v1 + (rotation_matrix1[0][1] * rotation_matrix2[0][1] +
#                rotation_matrix1[1][1] * rotation_matrix2[1][1] +
#                rotation_matrix1[2][1] * rotation_matrix2[2][1]) * AV2[1]  # yy
#     v1 = v1 + (rotation_matrix1[0][2] * rotation_matrix2[0][2] +
#                rotation_matrix1[1][2] * rotation_matrix2[1][2] +
#                rotation_matrix1[2][2] * rotation_matrix2[2][2]) * AV2[2]  # zz

#     v1 = v1 + (rotation_matrix1[0][0] * rotation_matrix2[0][1] +
#                rotation_matrix1[0][1] * rotation_matrix2[0][0] +
#                rotation_matrix1[1][0] * rotation_matrix2[1][1] +
#                rotation_matrix1[1][1] * rotation_matrix2[1][0] +
#                rotation_matrix1[2][0] * rotation_matrix2[2][1] +
#                rotation_matrix1[2][1] * rotation_matrix2[2][0]) * AV2[3]  # xy
#     v1 = v1 + (rotation_matrix1[0][0] * rotation_matrix2[0][2] +
#                rotation_matrix1[0][2] * rotation_matrix2[0][0] +
#                rotation_matrix1[1][0] * rotation_matrix2[1][2] +
#                rotation_matrix1[1][2] * rotation_matrix2[1][0] +
#                rotation_matrix1[2][0] * rotation_matrix2[2][2] +
#                rotation_matrix1[2][2] * rotation_matrix2[2][0]) * AV2[4]  # xz
#     v1 = v1 + (rotation_matrix1[0][1] * rotation_matrix2[0][2] +
#                rotation_matrix1[0][2] * rotation_matrix2[0][1] +
#                rotation_matrix1[1][1] * rotation_matrix2[1][2] +
#                rotation_matrix1[1][2] * rotation_matrix2[1][1] +
#                rotation_matrix1[2][1] * rotation_matrix2[2][2] +
#                rotation_matrix1[2][2] * rotation_matrix2[2][1]) * AV2[5]  # yz

#     return 2 * v1


# def calculateAVandAV2andNormL(str1):
#     '''
#     This one creates the vectors:
#         AV:     average_x, average_y, average_z
#         AV2:    xx, yy, zz, xy, xz, yz
#     and the NormL value
#     '''

#     average_x, average_y, average_z, total_atoms = 0, 0, 0, 0
#     xx, yy, zz, xy, xz, yz = 0, 0, 0, 0, 0, 0

#     for residue in str1.get_residues():
#         for atom in residue.get_atoms():
#             average_x = average_x + atom.get_x()
#             average_y = average_y + atom.get_y()
#             average_z = average_z + atom.get_z()

#             xx = xx + atom.get_x() * atom.get_x()
#             yy = yy + atom.get_y() * atom.get_y()
#             zz = zz + atom.get_z() * atom.get_z()
#             xy = xy + atom.get_x() * atom.get_y()
#             xz = xz + atom.get_x() * atom.get_z()
#             yz = yz + atom.get_y() * atom.get_z()

#             total_atoms += 1

#     average_x = average_x / float(total_atoms)
#     average_y = average_y / float(total_atoms)
#     average_z = average_z / float(total_atoms)

#     xx = xx / float(total_atoms)
#     yy = yy / float(total_atoms)
#     zz = zz / float(total_atoms)
#     xy = xy / float(total_atoms)
#     xz = xz / float(total_atoms)
#     yz = yz / float(total_atoms)

#     NormL = 2 * (xx + yy + zz)

#     return average_x, average_y, average_z, xx, yy, zz, xy, xz, yz, NormL



import os, sys
class ResidueDistanceError(Exception):
    '''
    The distance between two residues cannot be computed because CA or CB atoms are not found
    '''
    def __init__(self, c_type, residue1, residue2):
        '''
        Constructor
        '''
        self.c_type = c_type
        self.residue1 = residue1
        self.residue2 = residue2

    def __str__(self):
        return 'EXCEPTION: Cannot compute %s distance between residue %s and %s!\n' % (self.c_type.upper(), self.residue1, self.residue2)

class ResidueTypeShortError(Exception):
    '''
    Cannot converts the 3 letter type into 1 letter type (example: 'ALA' -> 'A')
    '''
    def __init__(self, residue_type):
        '''
        Constructor
        '''
        self.residue_type = residue_type

    def __str__(self):
        return '%s residue type has not a valid short translation\n' % self.residue_type

class ResidueExpositionError(Exception):
    '''
    The exposition of a residue cannot be computed
    '''
    def __init__(self, residue_num, reason):
        '''
        Constructor
        '''
        self.residue_num = residue_num
        self.reason = reason

    def __str__(self):
        return 'Cannot compute residue exposition for residue %s: %s' % (self.residue_num, self.reason)

class ResidueChargeError(Exception):
    '''
    The charge of a residue cannot be computed
    '''
    def __init__(self, residue_num):
        '''
        Constructor
        '''
        self.residue_num = residue_num

    def __str__(self):
        return 'Cannot compute charge for residue %s\n' % self.residue_num

class ResiduePolarityError(Exception):
    '''
    The Polarity of a residue cannot be computed
    '''
    def __init__(self, residue_num):
        '''
        Constructor
        '''
        self.residue_num = residue_num

    def __str__(self):
        return 'Cannot get polarity for residue %s\n' % self.residue_num

class ResidueTriadError(Exception):
    '''
    Cannot compute the triad for a residue
    '''
    def __init__(self, residue_num, reason):
        '''
        Constructor
        '''
        self.residue_num = residue_num
        self.reason = reason

    def __str__(self):
        return 'Cannot compute triad for residue %s: %s' % (self.residue_num, self.reason) 

class RelocateProgramError(Exception):
    '''
    Incorrect docking program used to relocate structures
    '''
    def __init__(self, docking):
        '''
        Constructor
        '''
        self.docking = docking

    def __str__(self):
        return 'Cannot relocate: Incorrect docking program %s\n' % self.docking


'''
@file: JSONer.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   04/2014

@ [oliva's lab](http://sbi.imim.es)

@class: JSONer
'''
import json
from abc import ABCMeta, abstractmethod


class JSONer(object):
    '''
    Children from this class have the json() method to be exported in json
    format.
    Every children must implement the as_dict() method that will be used to
    format every attribute in a way that is json-compatible.
    '''
    __metaclass__ = ABCMeta

    @abstractmethod
    def as_dict(self):
        '''
        Mandatory method to implement in children of the {JSONer} class.
        Describes the class as a dictionary with all the values compatible with
        the json format.

        @return: {Dictionary}
        '''
        raise NotImplementedError()

    def json(self, pretty=False):
        '''
        Transforms the data of the class into a json formated String.
        Requires as_dict() function to be implemented.

        @param:    pretty
        @pdef:     Whether or not to prettify the json.
        @pdefault: _False_
        @ptype:    {Boolean}

        @return: {String}
        '''
        if pretty:
            return json.dumps(self.as_dict(), indent=2, separators=(',', ':'))
        return json.dumps(self.as_dict(), separators=(',', ':'))



import sys, os, argparse, gzip

from SBI.beans           import Path

from archdb.src import start_transaction, end_transaction

# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'enrichment2SQL',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-i', '--input',          dest     = "inputfile", 
                        action = "store",         required = True,  help = "File of enrichments")
    parser.add_argument('-c', '--classification', dest     = "clust", choices = ['class','subclass'],
                        action = "store",         required = True,  help = "Class or Subclass")
    parser.add_argument('-r', '--relation',       dest     = "rel",   choices = ['scop','go','enzyme','drugBank'],
                        action = "store",         required = True,  help = "Class or Subclass")
    parser.add_argument('-v', '--verbose',        dest     = "verbose", 
                        action = "store_true",    default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    options.__dict__['table'] = "_".join([options.rel,options.clust,'enrichment'])
    return options

def enrichment2SQL(infile, table, rel, verbose):
   
    if verbose: sys.stderr.write("Retrieving data from {0} ...\n".format(infile))
    if verbose: sys.stderr.write("\tTo table {0} ...\n".format(table))
    start_command = "INSERT INTO {0} VALUES ".format(table)
    sys.stdout.write(start_transaction()+"\n")
    fd = open(infile)
    for line in fd:
        d = line.strip().split()
        d[1] = d[1] if rel in ['scop','go'] else "'"+d[1]+"'"
        sys.stdout.write("{0} ({1[0]},{1[1]},'{1[7]}',{1[2]},'{1[8]}','{1[9]}','{1[10]}');\n".format(start_command,d))
    sys.stdout.write(end_transaction())
 
    if verbose: sys.stderr.write("End execution.\n")

if __name__ == "__main__":
    options = options()
    enrichment2SQL(options.inputfile, options.table, options.rel, options.verbose)


import sys, os, argparse, gzip

from SBI.beans           import Path
from SBI.databases import GOlink

from archdb.src import Source, GOterm
from archdb.src import start_transaction, end_transaction

# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'GO2SQL',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--database',     dest     = "database", 
                        action = "store",       required = True,  help = "Destination Directory for GO database.")
    parser.add_argument('-s', '--sql',          dest     = "sqlfile", 
                        action = "store",       required = True,  help = "Output SQL file")
    parser.add_argument('-k', '--skipdownload', dest     = "skip_download", 
                        action = "store_true",  default  = False,  help = "Skips downloading the database.")
    parser.add_argument('-v', '--verbose',      dest     = "verbose", 
                        action = "store_true",  default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options

def GO2SQL(database, sqlfile, skip_download, verbose):
    go_connect = GOlink(local = database)
    newsource  = None
    if not skip_download:
        if verbose: sys.stderr.write("Downloading GO database to {0} ...\n".format(database))
        go_connect.download()
        newsource = Source(name   = 'GO', source = go_connect.source)
        if verbose: sys.stderr.write("Download Finished.\n")
    else:
        if verbose: sys.stderr.write("Using previously downloaded database.\n")

    with_parents   = []
    with_relations = []

    if verbose: sys.stderr.write("Parsing GO.\n")
    if verbose: sys.stderr.write("Writing {0} ....\n".format(sqlfile))
    Path.mkdir(os.path.split(os.path.abspath(sqlfile))[0])
    sql_fd = gzip.open(sqlfile, 'wb')
    sql_fd.write(start_transaction())
    if newsource is not None:
        sql_fd.write(newsource.toSQL())

    for go_line in go_connect.localGOs:
        newGO = GOterm(inline = go_line)
        sql_fd.write(newGO.toSQL() + "\n")
        if len(newGO.relations) > 0:
            with_relations.append(newGO)
        if len(newGO.parents) > 0:
            with_parents.append(newGO)

    for GO in with_relations:
        sql_fd.write(GO.relations2SQL() + "\n")
    for GO in with_parents:
        sql_fd.write(GO.parents2SQL() + "\n")

    sql_fd.write(end_transaction())
    sql_fd.close()
    if verbose: sys.stderr.write("End execution.\n")

if __name__ == "__main__":
    options = options()
    GO2SQL(options.database, options.sqlfile, options.skip_download, options.verbose)


'''
@file: BlastResult.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2011

@ [oliva's lab](http://sbi.imim.es)

@class: BlastResult
@class: BlastHeader
@class: HitFilter
'''
import os
import copy

import BlastExe
from SBI.beans              import StorableObject
from SBI.beans              import File
from SBI.sequence           import Sequence
from SBI.sequence.alignment import Rost
from SBI                    import SBIglobals as SBIg


class BlastResult(StorableObject):
    '''
    Contains all the information provided by a BLAST execution.
    Basically is a container of {BlastHit}.

    Theoretically, it will always be created through the {BlastExe} object.

    The length of the {BlastResult} correspond to the number of hits in the
    iteration of interest (last iteration by default)

    '''
    def __init__(self, query_name, query_sequence, header = None):
        '''
        @param:    query_name
        @pdef:     name of the query sequence
        @ptype:    {String}

        @param:    query_sequence
        @pdef:     query sequence
        @ptype:    {String}

        @param:    header
        @pdef:     main data about the blast execution
        @pdefault: _None_
        @ptype:    {BlastHeader}
        '''
        self._query  = Sequence(query_name, query_sequence)
        self._header = header
        self._filter = HitFilter()

        self._lastiteration = 0                  # Keep the last iteration
        self._hits          = []                 # List of BlastHit objects
        self._correctedHits = False

        self._filter_hits   = None
        self._query_index   = None

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def query(self):
        '''
        Name of the query sequence

        @return: {String}
        '''
        return self._query.full_identifier

    @property
    def query_length(self):
        '''
        Length of the query sequence

        @return: {Integer}
        '''
        return len(self._query)

    @property
    def query_sequence(self):
        '''
        Query sequence

        @return: {String}
        '''
        return self._query.sequence

    @property
    def query_object(self):
        '''
        {Sequence} query

        @return: {Sequence}
        '''
        return self._query

    @property
    def version(self):
        '''
        Version of the blast executable.

        @return: {String}
        '''
        return self._header.version

    @property
    def matrix(self):
        '''
        Similarity matrix used.

        @return: {String}
        '''
        return self._header.matrix

    @property
    def gap_open(self):
        '''
        Gap open penalty

        @return: {Integer}
        '''
        return self._header.gap_open

    @property
    def gap_extend(self):
        '''
        Gap extension penalty

        @return: {Integer}
        '''
        return self._header.gap_extend

    @property
    def database(self):
        '''
        Name of the database searched

        @return: {String}
        '''
        return self._header.database

    @property
    def lastiteration(self):
        '''
        Number of the last iteration.
        0 means that there are no hits found.

        @return: {Integer}
        '''
        return int(self._lastiteration)

    @property
    def hits(self):
        '''
        Blast hits contained in the {BlastResult}. According to certain
        selection parameters.

        @return: {List} of {BlastHit}
        '''
        if self._filter_hits is not None:
            return self._filter_hits

        self._filter_hits = []

        i = self._filter.iteration
        e = self._filter.evalue
        r = self._filter.tz_type
        p = self._filter.tz_parameter
        o = self._filter.overlap

        for hit in self.raw_hits:
            ros = (r is None or hit.evaluate_Rost_twilight_zone(r, p))
            ite = hit.iteration == i
            eva = (e is None or hit.evalue <= float(e))
            if ite and eva and ros:
                self._filter_hits.append(hit)
            if hit.iteration > i:  # ignore bigger iterations, if any
                break

        if o == 1 or len(self._filter_hits) == 0:
            return self._filter_hits
        else:
            return self._get_nonoverlaping_hits(self._filter_hits, o)

    @property
    def raw_hits(self):
        '''
        All {BlastHit} contained in the object.

        @return: {List} of {BlastHit}
        '''
        return self._hits

    ############
    # BOOLEANS #
    ############
    @property
    def are_hits_corrected(self):
        '''
        Checks if hits have already been corrected.

        @return: {Boolean}
        '''
        return self._correctedHits

    @property
    def is_selfhit_ignored(self):
        '''
        Checks if self hit has been ignored (if any)

        @return: {Boolean}
        '''
        return not self._header.self_hit

    @property
    def has_hits(self):
        '''
        Checks that there is some hit from the blast result

        @return: {Boolean}
        '''
        return not self.lastiteration == 0

    @property
    def is_empty(self):
        '''
        Checks that there are no hits from the blast result (no homologs)

        @return: {Boolean}
        '''
        return not self.has_hits

    ###########
    # METHODS #
    ###########
    def add_hit(self, hit):
        '''
        Add a new blast hit to the container.

        '''
        self._hits.append(hit)

    def set_hit_filter(self, iteration = None, evalue = None,
                       tz_type = None, tz_parameter = None, overlap = 1):
        '''
        Set parameters to exclude some hits that do not fulfill the filter.
        The parameters can be passed to the different functions that call the hits,
        but doing it here implies not having to do it every time.
        _IMPORTANT:_ Any non-specified parameter will be set to defaults.

        @param:    iteration
        @pdef:     query iteration. if a iteration bigger than the last iteration
                   is given, it defaults to last iteration. if 0 is given, the first
                   iteration (1) is assumed.
        @pdefault: last iteration
        @ptype:    {Integer}

        @param:    evalue
        @pdef:     maximum e-value considered.
        @pdefault: _None_, no filter by e-value
        @ptype:    {Float}

        @param:    tz_type
        @pdef:     Rost curve.
        @pdefault: _None_, no filter by Rost
        @poptions: 'ID', 'identity', 'SIM', 'similarity', 'hssp'
        @ptype:    {String}

        @param:    tz_parameter
        @pdef:     threshold parameter for Rost's curve
        @pdefault: 5 for 'ID', 'identity'; 12 for 'SIM', 'similarity';
                   8 for 'hssp'
        @pclash:   if tz_type is _None_ is ignored
        @ptype:    {Integer}

        @param:    overlap
        @pdef:     maximum amount of sequence overlap (over the query) allowed
                   between results. from 0 (no overlap) to 1 (total overlap).
                   Overlap values bigger than 1 default to 1, those smaller than
                   0 default to 0.
        @pdefault: 1, no filter by overlap.
        @ptype:    {Float}
        '''
        # TODO review overlap
        self.set_hit_filter_default()

        if iteration is not None:
            self._filter.iteration = iteration
            SBIg.alert('debug', self, 'Set iteration filter to {0}'.format(self._filter.iteration))
        if evalue is not None:
            self._filter.evalue = evalue
            SBIg.alert('debug', self, 'Set evalue filter to {0}'.format(self._filter.evalue))
        if tz_type is not None:
            self._filter.tz_type = tz_type
            SBIg.alert('debug', self, 'Activating {0} Rost'.format(self._filter.tz_type))
            if tz_parameter is not None:
                self._filter.tz_parameter = tz_parameter
                SBIg.alert('debug', self, 'Setting Rost at {0}'.format(self._filter.tz_parameter))
        if overlap != 1:
            self._filter.overlap = overlap
            SBIg.alert('debug', self, 'Set overlap filter to {0}'.format(self._filter.overlap))

    def set_hit_filter_default(self):
        '''
        Return the hit filter to default parameters: NO FILTER
        '''
        self._filter_hits = None
        self._filter.lastiteration = self._lastiteration
        self._filter.set_default()

    def get_hit(self, hit_ID):
        '''
        Get a hit by hit sequence ID.

        @param:    hit_ID
        @pdef:     name of the hit
        @ptype:    {String}
        '''
        for h in self.hits:
            if h.sequenceID == hit_ID:
                return h
        return None

    def get_used_hits(self):
        '''
        Retrieves all the {BlastHit} that have been marked as used.

        @return: {List} of {BlastHit}
        '''
        hits_of_interest = []
        for hit in self._hits:
            if hit.is_used:
                hits_of_interest.append(hit)
        return hits_of_interest

    def correct_hit_count(self, count_hit_file = None, count_query_file = None,
                          return_correction_dict = False):
        '''
        Corrects the starting point of the hits and the query, if needed.
        Why?
        When blasting vs. PDB (for example), sometimes the hit positions given
        by blast are wrong, as the blast always consider the first position of
        the hit sequence as 1 and PDB does not.
        Even more, the position reference doesn't even need to be a number.
        As the specific location in the PDB is important, we need to adapt our
        blasts so than we can read that data.
        Keep in mind that hits and query must be corrected together in this step,
        as this function cannot be called twice for a same instance.

        @param:    count_hit_file
        @pdef:     file containing the idex data for the query database
                   each sequence in this file will have a format such as:
                   >3K2K_A -7 ;-6 ;-5 ;-4 ;-3 ;-2 ;-1 ;0 ;1 ;2 ;3 ;4 ;5 ;6 ;7 ...
        @ptype:    {String}

        @param:    count_query_file
        @pdef:     sometimes we might also need to correct the query (if PDB vs.
                   PDB). Same format as count_hit_file. They might be the same file.
        @ptype:    {String}

        @param:    return_correction_dict
        @pdef:     instead of actually executing the correction, it only returns
                   the dictionary for further use.
        @pdefault: _False_
        @ptype:    {Boolean}

        @raises: {IOError} if the correction index file does not exist.
        @raises: {AttributeError} if the BlastResult does not contain any BlastHit.
        @raises: {BlastError} if it has been called before for this instance.

        '''
        if not self.has_hits:
            SBIg.warn(self, "BlastResult of {0} has no hits to correct".format(self.query))
            return

        if self.are_hits_corrected:
            be = BlastExe.BlastError()
            raise be.corrected_hits()

        SBIg.alert('debug', self, 'Correcting indexes for {0}'.format(self.query))
        cfile = File(count_hit_file)
        cq    = False

        codes_of_interest = set([hit.sequenceID for hit in self.raw_hits])
        if count_query_file == count_hit_file:
            codes_of_interest.add(self.query)
            count_query_file = None
            cq               = True

        start_index_dic = {}
        for line in cfile.read():
            if len(line.strip()) > 0:
                k = line.split('\t')
                if k[0].lstrip('>') in codes_of_interest:
                    start_index_dic[k[0].lstrip('>')] = k[1].strip().split(';')
        cfile.close()

        if count_query_file is not None:
            cfile = File(count_query_file)
            for line in cfile.read():
                if len(line.strip()) > 0:
                    k = line.split('\t')
                    if k[0].lstrip('>') == self.query:
                        start_index_dic[k[0].lstrip('>')] = k[1].strip().split(';')
            cfile.read().close()
            cq = True

        if cq:
            SBIg.alert('debug', self, '\tFixing Query {0}'.format(self.query))
            self._query_index = start_index_dic[self.query]

        if return_correction_dict:
            return start_index_dic

        for hit in self._hits:
            # This tests between the options PDB/PDB_ID or PDB_ID in case
            # the TAB file has different codification
            h      = hit.sequenceID
            hit_ID = h if h in start_index_dic else h.split("/")[-1]
            SBIg.alert('debug', self, '\tFixing {0}'.format(hit_ID))
            hit.correct_hit_count(new_index = start_index_dic[hit_ID])
            if cq:
                SBIg.alert('debug', self, '\tFixing Query {0}'.format(self.query))
                hit.correct_query_count(new_index = start_index_dic[self.query])

        self._correctedHits = True

    def set_last_iteration(self):
        '''
        Modifies the value of the last iteration with the value of iteration
        of the last BlastHit found

        '''
        self._lastiteration = 0 if self._hits == [] else self._hits[-1].iteration
        self._header.lastiteration = self._lastiteration

    def same_blast_version(self, blastObject):
        '''
        Are the two blasts the same executable?

        @param:    blastObject
        @pdef:     object against which compare
        @ptype:    {BlastResult}
        '''
        return self._header.same_version(blastObject._header)

    def same_blast_database(self, blastObject):
        '''
        Do the two blast use the same database?

        @param:    blastObject
        @pdef:     object against which compare
        @ptype:    {BlastResult}
        '''
        return self._header.same_database(blastObject._header)

    def same_blast_matrix(self, blastObject):
        '''
        Have two blast the same similarity matrix?

        @param:    blastObject
        @pdef:     object against which compare
        @ptype:    {BlastResult}
        '''
        return self._header.same_matrix(blastObject._header)

    def same_blast_iterations(self, blastObject):
        '''
        Have two blast the same number of iterations?

        @param:    blastObject
        @pdef:     object against which compare
        @ptype:    {BlastResult}
        '''
        return self.lastiteration == blastObject.lastiteration

    def same_blast_gap_properties(self, blastObject):
        '''
        Have two blast the same gap opening/extension policy?

        @param:    blastObject
        @pdef:     object against which compare
        @ptype:    {BlastResult}
        '''
        return self._header.same_gap_properties(blastObject._header)

    def comparable(self, blastObject):
        '''
        Are two blasts comparable?

        @param:    blastObject
        @pdef:     object against which compare
        @ptype:    {BlastResult}
        '''
        return self._header == blastObject._header

    def duplicate(self, new_query_sequence = None):
        '''
        Returns a copy of the object

        @param:    new_query_sequence
        @pdef:     New sequence to substitute the source query
        @ptype:    {Sequence}

        @return: {BlastResult}
        '''
        b = copy.deepcopy(self)
        if new_query_sequence is not None:
            b._query = new_query_sequence
        return b

    @staticmethod
    def build_fileobject_name(query_sequence, directory):
        '''
        Constructs a standardized name for the file to store the fileobject in.

        @param:    query_sequence
        @pdef:     query sequence
        @ptype:    {Sequence}

        @param:    directory
        @pdef:     global output directory
        @ptype:    {String}

        @return: {String}
        '''
        blastid  = '.'.join([query_sequence.md5, query_sequence.id, 'blastObj.gz'])
        blastdir = os.path.join(directory, os.path.join('blastOut', blastid[2:5]))
        return os.path.join(blastdir, blastid)

    def str_compacted_blast(self):
        '''
        Create a compacted representation of the alignment.

        @return: {String}
        '''
        lines = self.str_blast_header()
        lines.append("#Query Sequence:        {0}".format(self.query_sequence))
        for hit in self.hits:
            lines.append("\t".join((self.query, str(self.query_length), str(hit))))
        return "\n".join(lines)

    def str_PIR(self, result = 0):
        '''
        Create a specific hit as a PIR alignment.

        @param:    result
        @pdef:     numer-identifier of the {BlastHit} of interest
        @pdefault: 0. First hit.
        @ptype:    {Integer}
        '''
        lines = []
        hit = self.hits[result]

        #Counting in the possibility that the ID of the PDB hit is:
        #    PDB/2ERD_A
        hit_name  = hit.sequenceID.replace("/", "|").upper()
        hit_chain = hit_name.split("_")[-1]
        hit_id    = hit_name.split("|")[-1].split("_")[0]
        hit_sq    = hit.hit_seq + "*"
        q_sq      = hit.query_seq + "*"

        hs      = '>P1;{0}\nsequence:{0}:{1}:{2}:{3}:{2}:.:.:.:.'
        header1 = hs.format(self.query, hit.query_pos[0], '.', hit.query_pos[-1])
        header2 = hs.format(hit_id, hit.hit_pos[0], hit_chain, hit.hit_pos[-1])

        lines.append(header1)
        lines.extend([q_sq[i:i+60] for i in range(0, len(q_sq), 60)])
        lines.append(header2)
        lines.extend([hit_sq[i:i+60] for i in range(0, len(hit_sq), 60)])

        return "\n".join(lines)

    def str_blast_header(self, as_string = False):
        '''
        Returns a readable account of the blast execution main properties.

        @param:    as_string
        @pdef:     instead of returning each line as a position in an array,
                   it already returns everything as a string
        @pdefault: _False_
        @ptype:    {Boolean}

        @returns: {List} or {String}
        '''
        return self._header.represent(self.query, as_string)

    def str_representation(self, line_split = 160):
        '''
        Creates a representation of the coverage of each hit over the
        query sequence.

        @param:    line_split
        @pdef:     number of characters per line
        @pdefault: 160
        @ptype:    {Integer}
        '''
        names = []
        seqs  = []

        n = line_split

        names.append(self.query)
        line = ''.join(['*'] * self.query_length)
        seqs.append([line[i:i+n] for i in range(0, len(line), n)])
        for hit in self.hits:
            names.append(hit.sequenceID)
            sequence = []
            symbol   = ' '
            ini, end = hit.query_threshold_coord
            for i in range(self.query_length):
                if self._query_index is not None:
                    position = self._query_index[i]
                else:
                    position = i + 1
                if position == ini:
                    symbol = '*'
                sequence.append(symbol)
                if position == end:
                    symbol = ' '
            line = ''.join(sequence)
            seqs.append([line[i:i+n] for i in range(0, len(line), n)])

        data = []
        for j in range(len(seqs[0])):
            for i in range(len(names)):
                data.append('{0:<10}'.format(names[i]) + ' ' + seqs[i][j])
            data.append('')
        return "\n".join(data)

    def print_query_fasta(self):
        '''
        Gives the query in FASTA format.

        @return: {String}
        '''
        return self._query.format('FASTA')

    def print_compacted_blast(self, out_file = None):
        '''
        Print the compacted format of the blast hit.

        @param:    out_file
        @pdef:     file to print the blast data into.
        @pdefault: _None_
        @ptype:    {String}
        '''
        if out_file is not None:
            output = File(out_file, 'w')
            output.write("%s\n" % self.str_compacted_blast())
            output.close()
        else:
            print self.str_compacted_blast()

    def print_representation(self, line_split = 160, out_file = None):
        '''
        Print the alignment representation of the blast hit.

        @param:    line_split
        @pdef:     number of characters per line
        @pdefault: 160
        @ptype:    {Integer}

        @param:    out_file
        @pdef:     file to print the blast data into.
        @pdefault: _None_
        @ptype:    {String}
        '''
        if out_file is not None:
            output = File(out_file, 'w')
            output.write("%s\n" % self.str_representation(line_split))
            output.close()
        else:
            print self.str_representation(line_split)

    @staticmethod
    def read_compacted_blast(compacted_blast_file):
        '''
        Read data from a printed compacted blast into {BlastResult}.
        Not all options will be available in that new object.

        @param:    compacted_blast_file
        @pdef:     file of the compacted blast print
        @ptype:    {String}

        @return: {BlastResult}
        '''
        from BlastHit import BlastHit
        query_name, query_sequence     = None, None
        version, matrix, database      = None, None, None
        gap_open, gap_extend, self_hit = None, None, None

        br = None

        cbf = File(compacted_blast_file)
        for line in cbf.read():
            if line.startswith('#'):
                if line.startswith('#Query:'):
                    query_name = line.strip().split()[-1]
                if line.startswith('#Query Sequence:'):
                    query_sequence = line.strip().split()[-1]
                if line.startswith('#Blast Version:'):
                    version = line.strip().split()[-1]
                if line.startswith('#Search on matrix:'):
                    matrix = line.strip().split()[-1]
                if line.startswith('#Gap open penalty:'):
                    gap_open = line.strip().split()[-1]
                if line.startswith('#Gap extension penalty:'):
                    gap_extend = line.strip().split()[-1]
                if line.startswith('#Database searched:'):
                    database = line.strip().split()[-1]
                if line.startswith('#Self Hit is omitted:'):
                    self_hit = line.strip().split()[-1]
            else:
                if br is None:
                    if version is None:
                        bh = None
                    else:
                        bh = BlastHeader(version, matrix, gap_open, gap_extend,
                                         database, self_hit)
                    br = BlastResult(query_name, query_sequence, bh)
                d = line.strip().split()
                hit = BlastHit([d[2], d[3]], [d[8], d[9]],
                               [int(x) for x in d[10].split(',')[0].split(':')],
                               1, [d[4], d[5], d[6], d[7]])
                br.add_hit(hit)
        cbf.close()

        return br

    ###################
    # PRIVATE METHODS #
    ###################
    def _get_nonoverlaping_hits(self, hitlist, overlap):
        '''
        Filters a list of selected hits according to their overlap, prioritizing
        the order of the list.

        @param:    hitlist
        @pdef:     list of selected hits
        @ptype:    {List} of {BlastHit}

        @param:    overlap
        @pdef:     maximum amount of sequence overlap (over the query) allowed
                   between results. from 0 (no overlap) to 1 (total overlap).
        @ptype:    {Float}

        @return: {List} of {BlastHit}
        '''
        final_hits = []
        final_hits.append(hitlist[0])
        for x in xrange(1, len(hitlist)):
            accepted = True
            for y in xrange(0, len(final_hits)):
                if hitlist[x].overlap(final_hits[y]) > overlap:
                    accepted = False
                    break
            if accepted:
                final_hits.append(hitlist[x])

        self._filter_hits = final_hits
        return self._filter_hits

    #################
    # CLASS METHODS #
    #################
    def __len__(self):
        c = 0
        for hit in self.raw_hits:
            if hit.iteration == self._filter.iteration:
                c += 1
            if hit.iteration > self._filter.iteration:
                break
        return c

    def __str__(self):
        lines = self.str_blast_header()
        for hit in self._hits:
            lines.append("\t".join((self.query, str(self.query_length), str(hit))))
        return "\n".join(lines)


class BlastHeader(object):
    '''
    Stores the main data that identifies the blast execution. For comparisons and
    as a record.

    '''
    def __init__(self, version, matrix, gap_open, gap_extend, database, self_hit):
        '''
        @param:    version
        @pdef:     version of the BLAST executable
        @ptype:    {String}

        @param:    matrix
        @pdef:     name of the similarity matrix
        @ptype:    {String}

        @param:    gap_open
        @pdef:     cost of opening a gap
        @ptype:    {Integer}

        @param:    gap_extend
        @pdef:     cost of extending a gap
        @ptype:    {Integer}

        @param:    database
        @pdef:     file name of the database
        @ptype:    {String}

        @param:    self_hit
        @pdef:     match with itself has not been captured (if any)
        @ptype:    {Boolean} or {String} ('TRUE', 'FALSE')
        '''
        self.version    = version
        self.matrix     = matrix
        self.gap_open   = int(gap_open)
        self.gap_extend = int(gap_extend)
        self.database   = database
        if isinstance(self_hit, basestring):
            if self_hit.upper() == 'TRUE':
                self.self_hit = True
            else:
                self.self_hit = False
        else:
            self.self_hit     = self_hit

    ###########
    # METHODS #
    ###########
    def represent(self, query_name, as_string = False):
        '''
        Returns a readable account of the blast execution main properties.

        @param:    query_name
        @pdef:     name of the query sequence
        @ptype:    {String}

        @param:    as_string
        @pdef:     instead of returning each line as a position in an array,
                   it already returns everything as a string
        @pdefault: _False_
        @ptype:    {Boolean}

        @returns: {List} or {String}
        '''
        lines = []

        lines.append("#Query:                 {0}".format(query_name))
        lines.append("#Blast Version:         {0}".format(self.version))
        lines.append("#Search on matrix:      {0}".format(self.matrix))
        lines.append("#Gap open penalty:      {0}".format(self.gap_open))
        lines.append("#Gap extension penalty: {0}".format(self.gap_extend))
        lines.append("#Database searched:     {0}".format(self.database))
        lines.append("#Self Hit is omitted:   {0}".format(str(not self.self_hit).upper()))

        return '\n'.join(lines) if as_string else lines

    def same_version(self, other):
        '''
        Compares the version with another header.

        @param:    other
        @pdef:     header against which compare
        @ptype:    {BlastHeader}

        @return: {Boolean}
        '''
        return self.version == other.version

    def same_matrix(self, other):
        '''
        Compares the matrix with another header.

        @param:    other
        @pdef:     header against which compare
        @ptype:    {BlastHeader}

        @return: {Boolean}
        '''
        return self.matrix == other.matrix

    def same_gap_open(self, other):
        '''
        Compares the gap_open with another header.

        @param:    other
        @pdef:     header against which compare
        @ptype:    {BlastHeader}

        @return: {Boolean}
        '''
        return self.gap_open == other.gap_open

    def same_gap_extend(self, other):
        '''
        Compares the gap_extend with another header.

        @param:    other
        @pdef:     header against which compare
        @ptype:    {BlastHeader}

        @return: {Boolean}
        '''
        return self.gap_extend == other.gap_extend

    def same_gap_properties(self, other):
        '''
        Compares all gap properties with another header.

        @param:    other
        @pdef:     header against which compare
        @ptype:    {BlastHeader}

        @return: {Boolean}
        '''
        return self.same_gap_open(other) and self.same_gap_extend(other)

    def same_database(self, other):
        '''
        Compares the database with another header.

        @param:    other
        @pdef:     header against which compare
        @ptype:    {BlastHeader}

        @return: {Boolean}
        '''
        return self.database == other.database

    #################
    # CLASS METHODS #
    #################
    def __eq__(self, other):
        a = self.same_version(other) and self.same_matrix(other)
        b = self.same_gap_properties(other) and self.same_database(other)
        return a and b

    def __ne__(self, other):
        return not self.__eq__(other)


class HitFilter(object):
    '''
    Stores different parameters that can be used to filter among all the {BlastHit}
    contained inside the {BlastResult}.
    It is initialized with all the parameters set to values that will not produce
    any kind of filter.
    '''
    _EVALUE     = None
    _ROST_CURVE = None
    _ROST_PARAM = None
    _OVERLAP    = 1

    def __init__(self):
        self._iteration, self._evalue          = None, None
        self._rost_curve, self._rost_parameter = None, None
        self._overlap                          = None

        self._lastiteration = 1

        self.set_default()

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def iteration(self):
        return self._iteration

    @iteration.setter
    def iteration(self, value):
        try:
            iteration = int(value)
        except:
            iteration = self._lastiteration

        if iteration > self._lastiteration:
            iteration = self._lastiteration
        elif iteration == 0:
            iteration = 1

        self._iteration = iteration

    @property
    def lastiteration(self):
        '''
        Last iteration of the blast

        @return: {Integer}
        '''
        return int(self._lastiteration)

    @lastiteration.setter
    def lastiteration(self, value):
        self._lastiteration = int(value)
        self._iteration     = int(value)

    @property
    def evalue(self):
        return self._evalue

    @evalue.setter
    def evalue(self, value):
        self._evalue = float(value) if value is not None else None

    @property
    def tz_type(self):
        return self._rost_curve

    @tz_type.setter
    def tz_type(self, value):
        value = value.upper() if value is not None else None
        self._rost_curve = value if Rost.available_call(value) else None

    @property
    def tz_parameter(self):
        return self._rost_parameter

    @tz_parameter.setter
    def tz_parameter(self, value):
        self._rost_parameter = int(value) if value is not None else None

    @property
    def overlap(self):
        return self._overlap

    @overlap.setter
    def overlap(self, value):
        value = value if value >= 0 else 0
        self._overlap = value if value <= 1 else 1

    ###########
    # METHODS #
    ###########
    def set_default(self):
        '''
        Returns all the parameters to default, non-filtering mode.
        '''
        self._iteration      = self._lastiteration
        self._evalue         = HitFilter._EVALUE
        self._rost_curve     = HitFilter._ROST_CURVE
        self._rost_parameter = HitFilter._ROST_PARAM
        self._overlap        = HitFilter._OVERLAP



from scipy.stats import hypergeom as hyp
import numpy     as np


def accumulative_hypergeometric(k, n, K, N):
    '''
    [k]: SUCCESS IN THE CLUSTER
    [n]: SIZE OF THE CLUSTER
    [K]: SUCCESS IN POPULATION
    [N]: SIZE OF THE POPULATION
    '''
    k, n, K, N = int(k), int(n), int(K), int(N)
    sf = hyp.sf(k, N, K, n)
    if sf < 1:
        return sf + hyp.pmf(k, N, K, n)
    else:
        return 1 - hyp.cdf(k, N, K, n) + hyp.pmf(k, N, K, n)


def frequency(k, n):
    '''
    [k]: SELECTION
    [n]: POPULATION
    '''
    return float(k)/float(n)


def logodds(k, n, K, N):
    '''
    [k]: SUCCESS IN THE CLUSTER
    [n]: SIZE OF THE CLUSTER
    [K]: SUCCESS IN POPULATION
    [N]: SIZE OF THE POPULATION
    '''
    return np.log(frequency(k, n)/frequency(K, N))


def mutual_information(k, n, K, N):
    '''
    [k]: SUCCESS IN THE CLUSTER
    [n]: SIZE OF THE CLUSTER
    [K]: SUCCESS IN POPULATION
    [N]: SIZE OF THE POPULATION
    '''
    return frequency(k, N)*logodds(k, n, K, N)
